import QtQuick 2.4
import ".."
import Components 1.0
import TrueTouch 1.0

Item {
    id: root

    anchors { fill: parent ; leftMargin: 100 * main.screenScaleX ; topMargin: 10 * main.screenScaleY }

    state: main.isCSA ? "EllipticalStride" : machineinfo.productTypeString

    property int rpm
    property int watts
    property int contactHR
    property int telemetryHR
    property int fanSpeed
    property int buttonNumber
    property int strideAdc
    property int striderpm


    onWattsChanged: wattDisplay.value = watts
    onRpmChanged: rpmDisplay.value = rpm
    onContactHRChanged: contactHRDisplay.value = contactHR
    onTelemetryHRChanged: telemetryHRDisplay.value = telemetryHR
    onButtonNumberChanged: keypressDisplay.value = buttonNumber
    onStrideAdcChanged: strideAdcDisplay.value = strideAdc
    onStriderpmChanged: strideRpmDisplay.value = striderpm

    Connections {
        target: nucleus
        /* keyFunc is the button function */
        onKeypadButtonPressed: keypressDisplay.value = keyFunc
        onKeypadButtonReleased: keypressDisplay.value = 0
    }

    Connections {
        target: heartrate
        onTelemetryHeartRateChanged: telemetryHR = telemetryHeartRate
        onContactHeartRateChanged: contactHR = contactHeartRate
    }

    Connections {
        target: machinecontrol
        onEstopChanged: {
            estopDisplay.value = estop
            // stop change diagnostic speed control to zero target when safety key opened
            if(!estop)
                speedControl.value = 0
        }
        onPedalRPMChanged: {root.rpm = pedalRPM; root.striderpm = pedalRPM}
        onStrideLengthAdcActualChanged: strideAdc = machinecontrol.strideLengthAdcActual
    }
    Component.onCompleted: {
        if(machineinfo.productTypeString === "Treadmill") {
            if(machineinfo.systemConfig === "NUCLEUS") {
                nucleus.getStaticBrakeState()

            }
        }
        triggerWorkoutAutoRestartToggle.value = options.triggerWorkoutAutoRestart
    }

    Component.onDestruction: {
        machinecontrol.treadmillSpeedTarget = 0
        /* When exiting from this screen, engage the static brake if measured
         * speed is 0
         */
        if(machinecontrol.treadmillSpeedActual === 0) {
            machinecontrol.treadmillStaticBrakeEngage()
        }
    }

    Connections {
        target: main.isTft && main.state === "DiagnosticState" && root.visible && diagnostics_screen.item !== null && diagnostics_screen.item.currentSelectedIndex >= 0 ? keymapper : null

        // handle exiting the screen
        onBackPressed: {
            main.debugKeys("backPressed", "DiagnosticsOption")
            diagnostics_screen.item.unselectScreen()
        }
    }

    KeyedControlList {
        id: controlList
        listening: root.visible && diagnostics_screen.item !== null && diagnostics_screen.item.currentSelectedIndex >= 0
        listView: mode

        ToggleButtonLabeled {
            id: triggerWorkoutAutoRestartToggle
            labelString: qsTr("Trigger Automatic Workout Restart") + application.emptyString

            value: options.triggerWorkoutAutoRestart
            visible: true

            onToggled: {
                options.triggerWorkoutAutoRestart = value
                if(options.triggerWorkoutAutoRestart ) {
                    if (options.workoutSummaryTimeout > 45) {
                        options.workoutSummaryTimeout = 45
                    }
                } else if (options.workoutSummaryTimeout < options.defaults.workoutSummaryTimeoutDefault){
                    options.workoutSummaryTimeout = options.defaults.workoutSummaryTimeoutDefault
                }
            }
        }

        SliderControlLabeled{
            id: contactHRDisplay
            headingString: qsTr("Contact Heart Rate: ") + application.emptyString
            value:contactHR
            displayValue: value ? value + qsTr("BPM") : qsTr("NONE")
            sliderVisible: false
        }
        SliderControlLabeled{
            id: telemetryHRDisplay
            headingString: qsTr("Telemetry Heart Rate: ") + application.emptyString
            value:telemetryHR
            displayValue: value ? value + qsTr("BPM") : qsTr("NONE")
            sliderVisible: false
        }

        SliderControlLabeled{
            id: fanControl
            headingString: qsTr("Fan Speed Control") + application.emptyString
            //value: fanSpeed
            minValue: 0
            maxValue: 100
            onValueChanged: fan.fanSpeed = fanControl.value;
        }

        SliderControlLabeled{
            id: keypressDisplay
            headingString: qsTr("Keypress: ") + application.emptyString
            value:buttonNumber
            sliderVisible: false

        }

        SliderControlLabeled{
            id: rpmDisplay
            headingString: qsTr("Pedal RPM: ") + application.emptyString
            value:root.rpm
            sliderVisible: false

        }

        SliderControlLabeled{
            id: wattDisplay
            headingString: options.en957Enabled ? qsTr("Work: ") + application.emptyString : qsTr("Watts: ") + application.emptyString
            value:root.watts
            sliderVisible: false

        }

        ButtonControlLabeled {
            id: safetyCircuitControl
            width: 750 * main.screenScaleX
            height: ( visible ? 60 : 0 ) * main.screenScaleX

            labelString: qsTr("Safety Circuit Control") + application.emptyString

            /* Only show button on a nucleus system. */
            buttonEnabled: machineinfo.systemConfig === "NUCLEUS" &&
                           (statusString === "Engaged" ||
                            statusString === "Disengaged")

            statusString: if(machinecontrol.estop === true) {
                              return qsTr("Engaged") + application.emptyString
                          } else {
                              return qsTr("Disengaged") + application.emptyString
                          }

            onButtonPressed: {
                if(statusString === "Engaged") {
                    machinecontrol.treadmillSafetyCircuitOpen()
                } else if(statusString === "Disengaged") {
                    machinecontrol.treadmillSafetyCircuitClose()
                }
            }

            Connections {
                target: machinecontrol
                onSafetyCircuitDisengaged: safetyCircuitControl.statusString = qsTr("Disengaged") + application.emptyString;
                onSafetyCircuitDisengaging: safetyCircuitControl.statusString = qsTr("Disengaging") + application.emptyString;
                onSafetyCircuitEngaged: safetyCircuitControl.statusString = qsTr("Engaged") + application.emptyString;
                onSafetyCircuitEngaging: safetyCircuitControl.statusString = qsTr("Engaging") + application.emptyString;
            }
        }

        ButtonControlLabeled {
            id: staticBrakeToggle
            width: 750 * main.screenScaleX
            height: ( visible ? 60 : 0 ) * main.screenScaleX

            labelString: qsTr("Static Brake Toggle") + application.emptyString

            statusString: if(machinecontrol.isStaticBrakeEngaged === true) {
                              return qsTr("Engaged") + application.emptyString
                          } else {
                              return qsTr("Disengaged") + application.emptyString
                          }

            buttonEnabled: (machinecontrol.treadmillSpeedTarget === 0)

            onButtonPressed: {
                if(machinecontrol.isStaticBrakeEngaged === true) {
                    machinecontrol.treadmillStaticBrakeDisengage()
                } else {
                    machinecontrol.treadmillStaticBrakeEngage()
                }
            }

            Connections {
                target: machinecontrol
                onIsStaticBrakeEngagedChanged: {
                    if(isStaticBrakeEngaged === true) {
                        staticBrakeToggle.statusString = qsTr("Engaged") + application.emptyString
                    } else {
                        staticBrakeToggle.statusString = qsTr("Disengaged") + application.emptyString
                    }
                }
            }
        }

        SliderControlLabeled{
            id: speedControl
            property bool valueUpdateOnly: false
            headingString: qsTr("Belt Speed") + application.emptyString
            displayValue: (machinecontrol.treadmillSpeedActual/10.0).toFixed(1) + " / " + (value / 10.0).toFixed(1) + " MPH"
            minValue: 0
            maxValue: machineinfo.treadmillSpeedMaxMph
            onValueChanged: {
                if(valueUpdateOnly === false) {
                    machinecontrol.treadmillSpeedTarget = speedControl.value
                } else {
                    valueUpdateOnly = false
                }
            }

            Component.onCompleted: value = machinecontrol.treadmillSpeedTarget

            Connections {
                target: errorManager

                onNucleusFaultCleared: {
                    if(speedControl.value !== 0) {
                        speedControl.valueUpdateOnly = true
                        speedControl.value = machinecontrol.treadmillSpeedActual
                    }
                }
            }
        }

        SliderControlLabeled{
            id: inclineControl
            property bool manualControl: false
            property bool valueUpdateOnly: false
            headingString: qsTr("Incline") + application.emptyString
            value: 0
            displayValue: "(" +
                          machinecontrol.treadmillInclineADCActual +
                          ") / " +
                          (machinecontrol.treadmillInclinePercentGradeActual / 10).toFixed(1) +
                          " / " + (value / 10.0).toFixed(1) + " %"
            minValue: machineinfo.treadmillInclineMinPercentGrade
            maxValue: machineinfo.treadmillInclineMaxPercentGrade
            stepValue: 5
            onValueChanged: {
                if(valueUpdateOnly === false) {
                    machinecontrol.treadmillInclineTarget = inclineControl.value;
                } else {
                    valueUpdateOnly = false
                }
            }

            Component.onCompleted: value = machinecontrol.treadmillInclinePercentGradeActual
            Connections {
                target: inclineUpControl
                onButtonIsDownChanged: {
                    if(inclineUpControl.buttonIsDown == true ) {
                        inclineControl.manualControl = true
                    }
                }
            }
            Connections {
                target: inclineDownControl
                onButtonIsDownChanged: {
                    if(inclineDownControl.buttonIsDown == true ) {
                        inclineControl.manualControl = true
                    }
                }
            }

            Connections {
                target: machinecontrol
                onIsInclineMovingChanged: {
                    if(inclineControl.manualControl === true && machinecontrol.isInclineMoving === false) {
                        inclineControl.manualControl = false
                        inclineControl.valueUpdateOnly = true
                        inclineControl.value = machinecontrol.treadmillInclinePercentGradeActual
                    }
                }
                onSafetyCircuitDisengaging: {
                    inclineControl.valueUpdateOnly = true
                    inclineControl.value = machinecontrol.treadmillInclinePercentGradeActual
                }
            }
        }

        ButtonControlLabeled {
            id: inclineUpControl
            width: 750 * main.screenScaleX
            height: ( visible ? 60 : 0 ) * main.screenScaleX

            labelString: qsTr("Incline Force Up") + application.emptyString

            statusString: ""

            buttonEnabled: (safetyCircuitControl.statusString === "Engaged")

            onButtonIsDownChanged: {
                if(buttonIsDown) {
                    machinecontrol.treadmillInclineUp()
                } else {
                    machinecontrol.treadmillInclineStop()
                }
            }
        }

        ButtonControlLabeled {
            id: inclineDownControl
            width: 750 * main.screenScaleX
            height: ( visible ? 60 : 0 ) * main.screenScaleX

            labelString: qsTr("Incline Force Down") + application.emptyString

            statusString: ""

            buttonEnabled: (safetyCircuitControl.statusString === "Engaged")

            onButtonIsDownChanged: {
                if(buttonIsDown) {
                    machinecontrol.treadmillInclineDown()
                } else {
                    machinecontrol.treadmillInclineStop()
                }
            }
        }

        ToggleButtonLabeled{
            id: runModeToggle
            labelString: qsTr("Constant Power Mode") + application.emptyString

            onToggled: value == true ? machinecontrol.machineMode = MachineControl.MODE_CONSTANT_POWER : machinecontrol.machineMode = MachineControl.MODE_CONSTANT_TORQUE

            value: {
                switch(machinecontrol.machineMode){
                case MachineControl.MODE_CONSTANT_POWER: return true;
                case MachineControl.MODE_CONSTANT_TORQUE: return false;
                default: return false;

                }
            }
        }

        SliderControlLabeled{
            id: torqueControl
            headingString: qsTr("Torque Control") + application.emptyString
            minValue: 0
            maxValue: 470
            onValueChanged: {
                machinecontrol.machineMode = MachineControl.MODE_CONSTANT_TORQUE
                machinecontrol.workload = torqueControl.value;
            }
            //value: root.resistance
        }

        SliderControlLabeled{
            id: wattControl
            headingString: options.en957Enabled ? qsTr("Work Control") + application.emptyString : qsTr("Watt Control") + application.emptyString
            minValue: 0
            maxValue: 600
            onValueChanged: {
                machinecontrol.machineMode = MachineControl.MODE_CONSTANT_POWER
                machinecontrol.workload = wattControl.value
            }
            //value: root.resistance
        }

        SliderControlLabeled{
            id: estopDisplay
            headingString: qsTr("Safety Switch: ") + application.emptyString
            value: machinecontrol.estop
            displayValue: value ? qsTr("inserted") : qsTr("removed")
            sliderVisible: false
        }

        // Stride Adjust Elliptical
        SliderControlLabeled{
            id: strideAdjustControl
            headingString:qsTr("Stride Length") + application.emptyString
            minValue: machineinfo.adjustableStrideEllipticalMinStrideIn;
            maxValue: machineinfo.adjustableStrideEllipticalMaxStrideIn;
            onValueChanged: {
                machinecontrol.strideLengthTarget = value;
            }
            Component.onCompleted: value = machinecontrol.strideLengthTarget
        }
        SliderControlLabeled{
            id: strideTorqueControl
            headingString:qsTr("Torque Control") + application.emptyString
            minValue: 0
            maxValue: 1000
            onValueChanged: {
                machinecontrol.machineMode = MachineControl.MODE_CONSTANT_TORQUE
                machinecontrol.workload = strideTorqueControl.value
            }
            //value: root.resistance
        }
        SliderControlLabeled{
            id: strideWattsControl
            headingString: options.en957Enabled ? qsTr("Work Control") + application.emptyString : qsTr("Watt Control") + application.emptyString
            minValue: 0
            maxValue: 600
            onValueChanged: {
                machinecontrol.machineMode = MachineControl.MODE_CONSTANT_POWER
                machinecontrol.workload = strideWattsControl.value
            }
            //value: root.resistance
        }

        SliderControlLabeled{
            id: strideRpmDisplay
            headingString: qsTr("Pedal RPM: ") + application.emptyString
            value:root.rpm
            sliderVisible: false
        }

        SliderControlLabeled{
            id: strideAdcDisplay
            headingString: qsTr("Stride ADC value: ") + application.emptyString
            value:root.strideAdc
            sliderVisible: false
        }
    }

    ListView{
        id: mode
        spacing: -2 * main.screenScaleX
        anchors.horizontalCenter: root.horizontalCenter
        anchors.verticalCenter: root.verticalCenter
        width: root.width * 0.8
        height: root.height * 1.0
        model: controlList.model
        snapMode: ListView.SnapToItem
        interactive: false
        highlightRangeMode: ListView.ApplyRange
        preferredHighlightBegin: height/2 - 30
        preferredHighlightEnd: height/2 + 30
        highlightMoveVelocity: 10000
        currentIndex: controlList.curIndex
        highlight: Component {
            Item {
                BorderImage {
                    id: selectionCursor
                    visible: main.isTft && controlList.listening
                    border { left: 70 * main.screenScaleX; top: 5 * main.screenScaleY; bottom: 5 * main.screenScaleY; right: 70 * main.screenScaleX }
                    source: "../" + main.graphicsPath + "tft/maintenance_menus_cursor.png"
                    width: mode.currentItem ? mode.currentItem.navCursorWidth + 70 * main.screenScaleX : 0
                    height: 56 * main.screenScaleY
                    horizontalTileMode: BorderImage.Stretch
                }
            }
        }
    }

    Text {
        id: bottomHelpText
        visible: main.isTft && diagnostics_screen.item !== null && diagnostics_screen.item.currentSelectedIndex >= 0

        height: 48 * main.screenScaleY
        font { family: "arial"; pixelSize: 36 * main.screenScaleY; bold: true }
        color: "#6c6c6c"
        text: qsTr("use <img src='../graphics/tft/arrow_up.png' /><img src='../graphics/tft/arrow_down.png' /> to navigate, use <img src='../graphics/tft/arrow_left.png' /> <img src='../graphics/tft/arrow_right.png' /> to change value") + application.emptyString

        anchors.bottom: parent.bottom
        anchors.bottomMargin: 25 * main.screenScaleY
        anchors.horizontalCenter: parent.horizontalCenter
    }

    states: [
        State {
            name: "EllipticalStride"
            PropertyChanges {target: estopDisplay ; visible:false }
            PropertyChanges {target: rpmDisplay ; visible:false }
            PropertyChanges {target: wattDisplay ; visible:false }
            PropertyChanges {target: speedControl ; visible:false }
            PropertyChanges {target: inclineControl ; visible:false }
            PropertyChanges {target: runModeToggle ; visible: true }
            PropertyChanges {target: torqueControl ; visible:false }
            PropertyChanges {target: wattControl ; visible: false }
            PropertyChanges {target: strideAdjustControl; visible: true }
            PropertyChanges {target: strideTorqueControl; visible: !runModeToggle.value }
            PropertyChanges {target: strideWattsControl; visible: runModeToggle.value }
            PropertyChanges {target: strideRpmDisplay; visible: true }
            PropertyChanges {target: strideAdcDisplay; visible: true }
            PropertyChanges {target: staticBrakeToggle ; visible: false }            
            PropertyChanges {target: safetyCircuitControl ; visible: false }
            PropertyChanges {target: inclineUpControl ; visible: false }
            PropertyChanges {target: inclineDownControl ; visible: false }



        },
        State {
            name: "Elliptical"
            PropertyChanges {target: estopDisplay ; visible:false }
            PropertyChanges {target: rpmDisplay ; visible:true }
            PropertyChanges {target: wattDisplay ; visible:true }
            PropertyChanges {target: speedControl ; visible:false }
            PropertyChanges {target: inclineControl ; visible:false }
            PropertyChanges {target: runModeToggle ; visible: true }
            PropertyChanges {target: torqueControl ; visible:!runModeToggle.value}
            PropertyChanges {target: wattControl ; visible: runModeToggle.value}
            PropertyChanges {target: strideAdjustControl; visible: false }
            PropertyChanges {target: strideTorqueControl; visible: false }
            PropertyChanges {target: strideWattsControl; visible: false }
            PropertyChanges {target: strideRpmDisplay; visible: false }
            PropertyChanges {target: strideAdcDisplay; visible: false }
            PropertyChanges {target: staticBrakeToggle ; visible: false }
            PropertyChanges {target: safetyCircuitControl ; visible: false }
            PropertyChanges {target: inclineUpControl ; visible: false }
            PropertyChanges {target: inclineDownControl ; visible: false }
        },
        State {
            name: "Lateral Trainer"
            PropertyChanges {target: estopDisplay ; visible:false }
            PropertyChanges {target: rpmDisplay ; visible:true }
            PropertyChanges {target: wattDisplay ; visible:true }
            PropertyChanges {target: speedControl ; visible:false }
            PropertyChanges {target: inclineControl ; visible:false }
            PropertyChanges {target: runModeToggle ; visible: true }
            PropertyChanges {target: torqueControl ; visible:!runModeToggle.value}
            PropertyChanges {target: wattControl ; visible: runModeToggle.value}
            PropertyChanges {target: strideAdjustControl; visible: false }
            PropertyChanges {target: strideTorqueControl; visible: false }
            PropertyChanges {target: strideWattsControl; visible: false }
            PropertyChanges {target: strideRpmDisplay; visible: false }
            PropertyChanges {target: strideAdcDisplay; visible: false }
            PropertyChanges {target: staticBrakeToggle ; visible: false }
            PropertyChanges {target: safetyCircuitControl ; visible: false }
            PropertyChanges {target: inclineUpControl ; visible: false }
            PropertyChanges {target: inclineDownControl ; visible: false }
        },
        State {
            name: "Bike"
            PropertyChanges {target: estopDisplay ; visible:false }
            PropertyChanges {target: rpmDisplay ; visible:true }
            PropertyChanges {target: wattDisplay ; visible:true }
            PropertyChanges {target: speedControl ; visible:false }
            PropertyChanges {target: inclineControl ; visible:false }
            PropertyChanges {target: runModeToggle ; visible: true }
            PropertyChanges {target: torqueControl ; visible:!runModeToggle.value}
            PropertyChanges {target: wattControl ; visible: runModeToggle.value}
            PropertyChanges {target: strideAdjustControl; visible: false }
            PropertyChanges {target: strideTorqueControl; visible: false }
            PropertyChanges {target: strideWattsControl; visible: false }
            PropertyChanges {target: strideRpmDisplay; visible: false }
            PropertyChanges {target: strideAdcDisplay; visible: false }
            PropertyChanges {target: staticBrakeToggle ; visible: false }
            PropertyChanges {target: safetyCircuitControl ; visible: false }
            PropertyChanges {target: inclineUpControl ; visible: false }
            PropertyChanges {target: inclineDownControl ; visible: false }
        },
        State {
            name: "Treadmill"
            PropertyChanges {target: estopDisplay ; visible:true && machineinfo.systemConfig === "BRAINBOARD"}
            PropertyChanges {target: rpmDisplay ; visible:false }
            PropertyChanges {target: wattDisplay ; visible:false }
            PropertyChanges {target: speedControl ; visible:true }
            PropertyChanges {target: inclineControl ; visible:true }
            PropertyChanges {target: runModeToggle ; visible: false }
            PropertyChanges {target: torqueControl ; visible:false }
            PropertyChanges {target: wattControl ; visible:false }
            PropertyChanges {target: strideAdjustControl; visible: false }
            PropertyChanges {target: strideTorqueControl; visible: false }
            PropertyChanges {target: strideWattsControl; visible: false }
            PropertyChanges {target: strideRpmDisplay; visible: false }
            PropertyChanges {target: strideAdcDisplay; visible: false }
            PropertyChanges {target: staticBrakeToggle ; visible: true }
            PropertyChanges {target: safetyCircuitControl ; visible: true }
            PropertyChanges {target: inclineUpControl ; visible: true }
            PropertyChanges {target: inclineDownControl ; visible: true }

        }
    ]

}
