import QtQuick 2.4
import WorkoutManager 1.0
import Components 1.0

import "string_format.js" as Format
import "program_setup.js" as Fields
import "setup_params.js" as SetupJS

Item {
    id: root
    property alias model: myListView.model
    property QtObject timeItem
    property QtObject distItem
    property QtObject calItem
    property bool timeGoal: true
    property bool distGoal: false
    property int maxCalPerHr: 1600

    onModelChanged: if (root.model) SetupJS.modelPostProcessing()

    function setParameter(type, value) {
        return SetupJS.setParameter(type, value)
    }

    function validateEntries() {
        keypad.submit() // submit current item
        return SetupJS.validateEntries()
    }

    function recalcMaxSegments() {
        SetupJS.calculateMaxSegmentsFromTime()
    }

    // Validate data relationships when any value is changed
    Connections {
        target: root.model ? root.model : null
        ignoreUnknownSignals: true
        onDataChanged: {
            if(isTreadmill && calItem) // Treadmill calorie goal setup
                SetupJS.validateTreadCalorieGoal()
            if (!isTreadmill && calItem) {
                SetupJS.validateNonTreadCalorieGoal()
            }
        }
    }

    Connections {
        target: timeItem
        onDataChanged: {
            if(timeItem.dataValue > 0) {
                root.timeGoal = true
                root.distGoal = false
            }
        }
    }
    Connections {
        target: distItem
        onDataChanged: {
            if(distItem.dataValue > 0) {
                root.timeGoal = false
                root.distGoal = true
            }
        }
    }

    Item {
        id: calWarning

        visible: false
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 50 * main.screenScaleY
        anchors.left: parent.left
        anchors.leftMargin: 10 * main.screenScaleX
        width: 450 * main.screenScaleX
        height: 50 * main.screenScaleY

        Image{
            id: calCautionSymbol
            source: main.graphicsPath + "Icon_Warning.png"
            anchors.left: parent.left
            anchors.verticalCenter: parent.verticalCenter
        }

        Text {
            text: qsTr( "The calorie goal chosen is too high for selected workout time.") + application.emptyString
            wrapMode: Text.WordWrap
            font.pixelSize: 16 * main.screenScaleY
            anchors.left: calCautionSymbol.right
            anchors.verticalCenter: parent.verticalCenter
        }
    }

    ListView {
        id: myListView
        anchors.fill: parent
        interactive: false
        delegate: myDelegate

        onCurrentIndexChanged: {
            keypad.clear()
            var item = model.get(currentIndex)
            if(!item)
                return
            if(item.units === "%") {
                keypad.length = 3
                keypad.show_entry = true
                keypad.decimalPlaces = 1
            }
            else {
                keypad.length = SetupJS.getMaxDigit(item.maxValue) + item.precision
                keypad.decimalPlaces = item.precision
                keypad.show_entry = false
            }

            keypad.units = item.units
        }

        function getBaseFieldHeight(fieldType)
        {
            switch(fieldType)
            {
            case WorkoutProgram.MAX_SPEED_TYPE:
            case WorkoutProgram.MAX_INCLINE_TYPE:
            case WorkoutProgram.LEVEL_TYPE:
            case WorkoutProgram.SPEED_TYPE:
            case WorkoutProgram.RUN_SPEED_TYPE:
            case WorkoutProgram.WORK_SPEED_TYPE:
            case WorkoutProgram.WALK_SPEED_TYPE:
            case WorkoutProgram.REST_SPEED_TYPE:
            case WorkoutProgram.DISTANCE_TYPE:
            case WorkoutProgram.DISTANCE_QUICK_TYPE:
            case WorkoutProgram.SEGMENT_TYPE:
            case WorkoutProgram.TIME_TYPE:
            case WorkoutProgram.PACE_TYPE:
            case WorkoutProgram.WORKLOAD_TYPE:
            case WorkoutProgram.AGE_TYPE:
            case WorkoutProgram.WEIGHT_TYPE:
            case WorkoutProgram.GENDER_TYPE:
            case WorkoutProgram.INCLINE_TYPE:
            case WorkoutProgram.CALORIES_TYPE:
            case WorkoutProgram.MIN_STRIDE_TYPE:
            case WorkoutProgram.MAX_STRIDE_TYPE:
            case WorkoutProgram.ELEVATION_TYPE:
                return 80 * main.screenScaleY;

            case WorkoutProgram.WORK_HEARTRATE_TYPE:
            case WorkoutProgram.REST_HEARTRATE_TYPE:
            case WorkoutProgram.TARGET_HEARTRATE_TYPE:
            case WorkoutProgram.HEARTRATEREADONLY_TYPE:
            case WorkoutProgram.FIXED_DISTANCE_TYPE:
            case WorkoutProgram.TIMEREADONLY_TYPE:
            case WorkoutProgram.CALORIE_POTENTIAL_TYPE:
                return 120 * main.screenScaleY;
            }
        }

        function getBackgroundSource(fieldType)
        {
            switch(fieldType)
            {
            case WorkoutProgram.MAX_SPEED_TYPE:
            case WorkoutProgram.MAX_INCLINE_TYPE:
            case WorkoutProgram.LEVEL_TYPE:
            case WorkoutProgram.SPEED_TYPE:
            case WorkoutProgram.RUN_SPEED_TYPE:
            case WorkoutProgram.WORK_SPEED_TYPE:
            case WorkoutProgram.WALK_SPEED_TYPE:
            case WorkoutProgram.REST_SPEED_TYPE:
            case WorkoutProgram.DISTANCE_TYPE:
            case WorkoutProgram.DISTANCE_QUICK_TYPE:
            case WorkoutProgram.SEGMENT_TYPE:
            case WorkoutProgram.TIME_TYPE:
            case WorkoutProgram.PACE_TYPE:
            case WorkoutProgram.WORKLOAD_TYPE:
            case WorkoutProgram.AGE_TYPE:
            case WorkoutProgram.WEIGHT_TYPE:
            case WorkoutProgram.GENDER_TYPE:
            case WorkoutProgram.INCLINE_TYPE:
            case WorkoutProgram.CALORIES_TYPE:
            case WorkoutProgram.MIN_STRIDE_TYPE:
            case WorkoutProgram.MAX_STRIDE_TYPE:
            case WorkoutProgram.ELEVATION_TYPE:
                return main.graphicsPath + "setup/Setup_Small_Down.png"

            case WorkoutProgram.WORK_HEARTRATE_TYPE:
            case WorkoutProgram.REST_HEARTRATE_TYPE:
            case WorkoutProgram.TARGET_HEARTRATE_TYPE:
            case WorkoutProgram.TARGET_HEARTRATEREADONLY_TYPE:
            case WorkoutProgram.FIXED_DISTANCE_TYPE:
            case WorkoutProgram.TIMEREADONLY_TYPE:
            case WorkoutProgram.CALORIE_POTENTIAL_TYPE:
                return main.graphicsPath + "setup/Setup_Large_Down.png"
            }
        }

        Keypad {
            id: keypad

            property variant model: myListView.model
            property int currentFieldType: getFieldType()

            function getFieldType() {
                if(!model)
                    return -1
                var field = model.get(myListView.currentIndex)
                if(!field)
                    return -1
                return field.fieldType
            }

            width: 320 * main.screenScaleX
            anchors.right: parent.right
            anchors.rightMargin: 500 * main.screenScaleX
            anchors.verticalCenter: parent.verticalCenter
            anchors.verticalCenterOffset: ( show_entry ? 0 : 50 ) * main.screenScaleY
            show_entry: false
            allow_close: false

            visible: !(currentFieldType == WorkoutProgram.GENDER_TYPE || currentFieldType == WorkoutProgram.FIXED_DISTANCE_TYPE || currentFieldType == WorkoutProgram.ELEVATION_TYPE)
            enabled: !(currentFieldType == WorkoutProgram.TARGET_HEARTRATEREADONLY_TYPE || currentFieldType == WorkoutProgram.TIMEREADONLY_TYPE)  // don't allow changing read-only field

            onValueEntered: {
                if(keypad.value && keypad.value != "0.0") {
                    var item = model.get(mainProgItem.selectedIndex)
                    var tmp
                    mainProgItem.valueEntered = true;
                    if(item.units === "%"){
                        tmp = workoutProgram.tableIndexOfClosestValue(workoutProgram.inclineTable,keypad.value*10)
                        console.debug(tmp + " index")
                        model.setProperty(mainProgItem.selectedIndex, "dataValue", tmp);
                    } else if (SetupJS.isSpeedType(item.fieldType) && options.metricUnits) {
                        // when the user is using metric, find the closest possible mph value in the speed table to correct their entry
                        tmp = Globals.kmToMile(parseFloat(value))
                        tmp = workoutProgram.tableClosestValue(workoutProgram.speedTable, Math.round(tmp*10))
                        if (isNaN(tmp))
                            tmp = workoutProgram.speedTable[0]
                        model.setProperty(mainProgItem.selectedIndex, "dataValue", tmp/10)
                    } else if (SetupJS.isStrideType(item.fieldType) && options.metricUnits) {
                        tmp = Globals.cmToInch(parseFloat(value))
                        tmp = Math.round(tmp)
                        model.setProperty(mainProgItem.selectedIndex, "dataValue", tmp)
                    }
                    else
                        model.setProperty(mainProgItem.selectedIndex, "dataValue", keypad.value);
                    mainProgItem.valueEntered = false;

                    var corrections = SetupJS.validateParam(mainProgItem.selectedIndex)
                    if (item.fieldType === WorkoutProgram.SEGMENT_TYPE) {
                        mainProgItem.numSegs = model.get(mainProgItem.selectedIndex).dataValue
                    }
                    mainProgItem.valueEntered = false;

                    // go to next field when enter pressed, if entry was not corrected by validation
                    if(corrections == 0) {
                        myListView.incrementCurrentIndex()
                        mainProgItem.selectedIndex = myListView.currentIndex
                        mainProgItem.fieldIndex = myListView.currentIndex+1;
                    }
                }
            }
            onKeyPress: {
                if (key != "ENTER") {
                    if(model.get(mainProgItem.selectedIndex).units === "%"){
                        ;
                    }
                    else {
                        if (keypad.value) {
                            var item = model.get(mainProgItem.selectedIndex)
                            var val = keypad.value
                            if (SetupJS.isSpeedType(item.fieldType) && options.metricUnits) {
                                val = Globals.kmToMile(parseFloat(val))
                            } else if (SetupJS.isStrideType(item.fieldType) && options.metricUnits) {
                                val = Globals.cmToInch(parseFloat(val))
                                val = Math.round(val)
                            }
                            model.setProperty(mainProgItem.selectedIndex, "dataValue", val);

                            if (item.fieldType === WorkoutProgram.TIME_TYPE) {
                                SetupJS.calculateMaxSegmentsFromTime()
                            }

                        }
                    }
                }
            }
        }

        Text {
            id: prog_setup_desc_text
            anchors.top: parent.top
            anchors.topMargin: 200 * main.screenScaleY
            anchors.right: parent.right
            anchors.rightMargin: 10 * main.screenScaleX
            width: 350 * main.screenScaleX
            text: {
                if (programIndex >= WorkoutProgram.MANUAL_PROG && programIndex < WorkoutProgram.SAVED_WORKOUT) {
                    // Subtract 3 from programIndex to account for NO_PROG and QUICK_START_PROG and TREAD_QUICK_START_PROG
                    var str = Fields.workout_programs()[(programIndex - 3)][(fieldIndex - 1)]

                    var trans = str ? (application.translateText("Setup", str) + application.emptyString) : ""
                    // CAN-746 Workout setup time text doesn't match max. workout time setting
                    if (str === Fields.prog_setup_time_99_desc || str === Fields.prog_setup_manual_time_desc || str === Fields.prog_setup_time_99_manual_desc || str === Fields.prog_setup_time_60_desc) {
                        trans = trans.replace("99", options.workoutTimeMax)
                    }

                    return trans
                } else {
                    return "NA"
                }
            }
            color: "black"
            wrapMode: Text.WordWrap
            lineHeight: 1.3
            horizontalAlignment: Text.AlignRight
            font.pixelSize: text.length > 300 * main.screenScaleX ? 18 * main.screenScaleY : 20 * main.screenScaleY
        }

        Component {
            id: myDelegate
            Item {
                id: dataElement

                property real detailsOpacity : 0
                property real levelDetailsOpacity : 0
                property real expandedDetails1Opacity : 0
                property real expandedDetails2Opacity : 0
                property real segmentDetailsOpacity : 0
                property real elevationDetailsOpacity: 0
                property string expandedState: getExpandedState(fieldType)
                property string closedState: getClosedState(fieldType)
                property alias _slider3: sliderObject3

                function getExpandedState(fieldtype) {
                    switch(fieldtype)
                    {
                    case WorkoutProgram.MAX_SPEED_TYPE:
                    case WorkoutProgram.MAX_INCLINE_TYPE:
                    case WorkoutProgram.SPEED_TYPE:
                    case WorkoutProgram.RUN_SPEED_TYPE:
                    case WorkoutProgram.WORK_SPEED_TYPE:
                    case WorkoutProgram.WALK_SPEED_TYPE:
                    case WorkoutProgram.REST_SPEED_TYPE:
                    case WorkoutProgram.TIME_TYPE:
                    case WorkoutProgram.PACE_TYPE:
                    case WorkoutProgram.WORKLOAD_TYPE:
                    case WorkoutProgram.AGE_TYPE:
                    case WorkoutProgram.WEIGHT_TYPE:
                    case WorkoutProgram.DISTANCE_TYPE:
                    case WorkoutProgram.INCLINE_TYPE:
                    case WorkoutProgram.CALORIES_TYPE:
                    case WorkoutProgram.MIN_STRIDE_TYPE:
                    case WorkoutProgram.MAX_STRIDE_TYPE:
                        return "Details"
                    case WorkoutProgram.ELEVATION_TYPE:
                        return "ElevationDetails"
                    case WorkoutProgram.LEVEL_TYPE:
                        return isTreadmill ? "LevelDetails" : "Details"
                    case WorkoutProgram.GENDER_TYPE:
                        return "GenderDetails"
                    case WorkoutProgram.DISTANCE_QUICK_TYPE:
                        return "ExpandedDetails1"
                    case WorkoutProgram.WORK_HEARTRATE_TYPE:
                    case WorkoutProgram.REST_HEARTRATE_TYPE:
                    case WorkoutProgram.TARGET_HEARTRATE_TYPE:
                        return "ExpandedDetails2"
                    case WorkoutProgram.TARGET_HEARTRATEREADONLY_TYPE:
                        return "Expanded2"
                    case WorkoutProgram.CALORIE_POTENTIAL_TYPE:
                    case WorkoutProgram.FIXED_DISTANCE_TYPE:
                    case WorkoutProgram.TIMEREADONLY_TYPE:
                        return "Expanded3"
                    case WorkoutProgram.SEGMENT_TYPE:
                        return "SegmentDetails"
                    }
                }

                function getClosedState(fieldtype) {
                    switch(fieldtype)
                    {
                    case WorkoutProgram.WORK_HEARTRATE_TYPE:
                    case WorkoutProgram.REST_HEARTRATE_TYPE:
                    case WorkoutProgram.TARGET_HEARTRATE_TYPE:
                    case WorkoutProgram.TARGET_HEARTRATEREADONLY_TYPE:
                        return "Expanded2"
                    case WorkoutProgram.FIXED_DISTANCE_TYPE:
                    case WorkoutProgram.TIMEREADONLY_TYPE:
                    case WorkoutProgram.CALORIE_POTENTIAL_TYPE:
                        return "Expanded3"
                    default:
                        return ""
                    }
                }

                state: ListView.isCurrentItem ? expandedState : closedState
                width: 425 * main.screenScaleX
                height: myListView.getBaseFieldHeight(model.fieldType)
                x: 10 * main.screenScaleX

                Image {
                    id: background
                    anchors.left: parent.left
                    anchors.leftMargin: 5 * main.screenScaleX
                    anchors.top: parent.top
                    anchors.topMargin: 10 * main.screenScaleY
                    width: 425 * main.screenScaleX
                    height: dataElement.height
                    source: myListView.getBackgroundSource(model.fieldType)
                    z: 1

                    MouseArea {
                        anchors.fill: parent
                        onClicked: {
                            // validate a field as it is exited
                            SetupJS.validateParam(myListView.currentIndex)

                            var item = myListView.model.get(mainProgItem.selectedIndex)
                            if (SetupJS.isSpeedType(item.fieldType) && options.metricUnits) {
                                // when the user is using metric, find the closest possible mph value in the speed table to correct their entry
                                var tmp
                                tmp = parseFloat(item.dataValue)
                                tmp = workoutProgram.tableClosestValue(workoutProgram.speedTable, Math.round(tmp*10))
                                if (isNaN(tmp))
                                    tmp = workoutProgram.speedTable[0]
                                myListView.model.setProperty(mainProgItem.selectedIndex, "dataValue", tmp/10)
                            }

                            // change to item when clicked, unless it is a read-only type
//                            if(fieldType != WorkoutProgram.HEARTRATEREADONLY_TYPE && fieldType != WorkoutProgram.FIXED_DISTANCE_TYPE)
                                myListView.currentIndex = index

                            // follow selected item
                            mainProgItem.selectedIndex = indexValue.text;
                            mainProgItem.fieldIndex = model.fieldIndex;
                        }
                    }
                }
                // folded up state elements
                Text {
                    id: itemName
                    anchors.top: parent.top
                    anchors.topMargin: 24 * main.screenScaleY
                    anchors.left: parent.left
                    anchors.leftMargin: 20 * main.screenScaleX
                    text: qsTranslate("WorkoutSetup", SetupJS.getItemLabel(model)) + SetupJS.getItemAdditionalLabel(model) + application.emptyString
                    font.pointSize: 20 * main.screenScaleY
                    font.bold: true
                    smooth: true
                    color: "black"
                    z: 2
                }

                Text {
                    id: itemUnits
                    anchors.top: parent.top
                    anchors.topMargin: 32 * main.screenScaleY
                    anchors.right: parent.right
                    anchors.rightMargin: 20 * main.screenScaleX
                    text: model.units
                    font.pointSize: 14 * main.screenScaleY
                    wrapMode: Text.WordWrap
                    font.bold: true
                    smooth: true
                    color: "black"
                    z: 2
                }
                Text {
                    id: itemValue
                    anchors.bottom: itemUnits.bottom
                    anchors.bottomMargin: -2 * main.screenScaleY
                    anchors.right: itemUnits.left
                    anchors.rightMargin: 5 * main.screenScaleX
                    text: SetupJS.getValueString(model)
                    font.pointSize: 20 * main.screenScaleY
                    font.bold: true
                    smooth: true
                    color: "black"
                    z: 2
                }
                // expanded state elements
                Text {
                    id: itemUnitsDetails
                    anchors.top: parent.top
                    anchors.topMargin: 87 * main.screenScaleY
                    anchors.right: parent.right
                    anchors.rightMargin: 15 * main.screenScaleX
                    text: model.units
                    font.pointSize: 20 * main.screenScaleY
                    wrapMode: Text.WordWrap
                    font.bold: true
                    smooth: true
                    color: "black"
                    z: 2
                    visible: false
                }
                ImageButton {
                    id: hrChartButton
                    anchors.top: parent.top
                    anchors.topMargin: 15 * main.screenScaleY
                    anchors.right: parent.right
                    anchors.rightMargin: -60 * main.screenScaleX
                    downimage: main.graphicsPath + "224/Data_HRC_Cruise_Down.png"
                    upimage: main.graphicsPath + "224/Data_HRC_Cruise_Up.png"
                    icon: ""
                    z: 2
                    visible: {
                        if (model.fieldType == WorkoutProgram.WORK_HEARTRATE_TYPE || model.fieldType == WorkoutProgram.REST_HEARTRATE_TYPE || model.fieldType == WorkoutProgram.TARGET_HEARTRATE_TYPE || model.fieldType == WorkoutProgram.TARGET_HEARTRATEREADONLY_TYPE) {
                            return true;
                        }
                        else {
                            return false;
                        }
                    }

                    onClicked: {
                        showHRChart();
                    }
                }
                Text {
                    // Value seen while editing expanded item
                    id: itemValueDetails
                    anchors.bottom: itemUnitsDetails.bottom
                    anchors.bottomMargin: -4 * main.screenScaleY
                    anchors.right: itemUnitsDetails.left
                    anchors.rightMargin: 5 * main.screenScaleX
                    text: SetupJS.getValueString(model)

                    font.pointSize: 34 * main.screenScaleY
                    font.bold: true
                    smooth: true
                    color: "black"
                    z: 2
                    visible: false
                    onTextChanged: {
                        SetupJS.itemValueDetailsTextChanged(sliderObject, sliderObject2, sliderObject3, genderSlider, model)
                    }
                }

                Text {
                    id: hrDividerText
                    anchors.top: parent.top
                    anchors.topMargin: 72 * main.screenScaleY
                    anchors.horizontalCenter: parent.horizontalCenter
                    anchors.horizontalCenterOffset: 40 * main.screenScaleX
                    text: "<Center>/<br>"+qsTr("OR")+"</Center>" + application.emptyString
                    font.pointSize: 14 * main.screenScaleY
                    font.bold: true
                    smooth: true
                    color: "black"
                    z: 2
                    visible: false
                }

                Text {
                    id: hrPercentageUnits
                    anchors.top: parent.top
                    anchors.topMargin: 83 * main.screenScaleY
                    anchors.right: parent.right
                    anchors.rightMargin: 20 * main.screenScaleX
                    text: qsTr("%MAX") + application.emptyString
                    font.pointSize: text.length > 6 * main.screenScaleX ? 12 * main.screenScaleY : 18 * main.screenScaleY
                    wrapMode: Text.WordWrap
                    width: 80 * main.screenScaleX
                    height: 40 * main.screenScaleY
                    verticalAlignment: Text.AlignBottom
                    font.bold: true
                    smooth: true
                    color: "black"
                    z: 2
                    visible: false
                }

                Text {
                    id: hrPercentageValue
                    anchors.bottom: hrPercentageUnits.bottom
                    anchors.bottomMargin: -6 * main.screenScaleY
                    anchors.right: hrPercentageUnits.left
                    anchors.rightMargin: 5 * main.screenScaleX
                    text: model.heartRatePercentage
                    font.pointSize: 32 * main.screenScaleY
                    font.bold: true
                    smooth: true
                    color: "black"
                    z: 2
                    visible: false
                }

                Text {
                    id: indexValue
                    text: model.selectedIndex
                    font.pointSize: 24 * main.screenScaleY
                    color: "black"
                    visible: false
                }

                Item {
                    // slider control box seen after expanding most simple items (age, weight, time, etc.)
                    id: details
                    enabled: visible && opacity > 0

                    property int value: sliderObject.value

                    height: 155 * main.screenScaleY
                    width: 460 * main.screenScaleX
                    anchors.top: dataElement.top
                    anchors.topMargin: 85 * main.screenScaleY
                    anchors.left: dataElement.left
                    anchors.leftMargin: 50 * main.screenScaleX
                    opacity: dataElement.detailsOpacity

                    Image {
                        id: dataIndicatorBackground
                        anchors.horizontalCenter: parent.horizontalCenter
                        anchors.verticalCenter: parent.verticalCenter
                        source: main.graphicsPath + "setup/Setup_Large_Up_Step2.png"
                    }

                    SliderControl {
                        id: sliderObject

                        repeat: true
                        valueProxy: model.fieldType !== WorkoutProgram.ELEVATION_TYPE ? model.dataValue : 0.0

                        Component.onCompleted: {
                            if (model.fieldType === WorkoutProgram.ELEVATION_TYPE) {
                                // Avoid warning about assigning QPointF to double
                                return
                            }
                            value = model.dataValue
                        }

                        onValueChanged: {

                            if (model.fieldType === WorkoutProgram.ELEVATION_TYPE) {
                                // Avoid warning about assigning QPointF to double
                                return
                            }
                            SetupJS.sliderObjectValueChanged(model, value, valueProxyChange, index)

                            // Limit speed/incline ranges for Alpine Runner
                            if (main.isAlpineRunner) {
                                if (model.fieldType === WorkoutProgram.SPEED_TYPE
                                        || model.fieldType === WorkoutProgram.INCLINE_TYPE
                                        || model.fieldType === WorkoutProgram.MAX_SPEED_TYPE
                                        || model.fieldType === WorkoutProgram.MAX_INCLINE_TYPE) {
                                    SetupJS.setAlpineCustomMaxValue(fieldType, value)
                                }
                            }
                        }
                        showValue: false
                        maxValue: model.maxValue
                        onMaxValueChanged: {
                            if (model.fieldType === WorkoutProgram.SPEED_TYPE || model.fieldType === WorkoutProgram.INCLINE_TYPE) {
                                sliderObject.valueChanged()
                            }
                        }

                        minValue: model.minValue
                        value: model.dataValue
                        precision: model.precision
                        stepValue: {
                            if (model.precision == 0) {
                                return 1;
                            }
                            else if (model.precision == 1) {
                                return 0.1;
                            }
                        }

                        anchors.top: parent.top
                        anchors.topMargin: 75 * main.screenScaleY
                        anchors.left: parent.left
                        anchors.leftMargin: 45 * main.screenScaleX
                        width: 350 * main.screenScaleX
                        height: 34 * main.screenScaleY
                    }
                }
                Item {
                    id: elevationDetails

                    height: 175 * main.screenScaleY
                    width: 1158 * main.screenScaleX
                    anchors.top: dataElement.top
                    anchors.topMargin: 115 * main.screenScaleY
                    anchors.left: dataElement.left
                    anchors.leftMargin: 50 * main.screenScaleX
                    opacity: dataElement.elevationDetailsOpacity

                    Image {
                        id: elevationBgImage
                        anchors.horizontalCenter: parent.horizontalCenter
                        anchors.verticalCenter: parent.verticalCenter
                        source: main.graphicsPath + "setup/alpine_setup_sliderbkgrd2.png"
                    }


                    DualSlider {
                        id: elevationSlider
                        property bool completed: false
                        enabled: visible && elevationDetails.opacity > 0

                        ListModel {
                            id: mtEverestIndexPoints
                            // NOTE: Left untranslated on purpose
                            ListElement { value: 0; label: "SEA LEVEL"; offset: 3}
                            ListElement { value: 17000; label: "BASE CAMP" }
                            ListElement { value: 19900; label: "LHO LA"; offset: -3}
                            ListElement { value: 21300; label: "CAMP 2"; offset: 6}
                            ListElement { value: 24500; label: "GENEVA SPUR" }
                            ListElement { value: 26000; label: "CAMP 4"; offset: 20}
                            ListElement { value: 29029; label: "SUMMIT" }
                        }
                        ListModel {
                            id: mtRainierIndexPoints
                            ListElement { value: 0; label: "SEA LEVEL"; offset: 3}
                            ListElement { value: 5440; label: "ALTA VISTA"; offset: 3}
                            ListElement { value: 10188; label: "CAMP MUIR"; offset: 3}
                            ListElement { value: 11600; label: "DISAPPOINTMENT\nCLEAVER"; offset: 3 }
                            ListElement { value: 14411; label: "SUMMIT" }
                        }
                        ListModel {
                            id: mtKilimanjaroIndexPoints
                            ListElement { value: 0; label: "SEA LEVEL"; offset: 3}
                            ListElement { value: 5905; label: "MACHAME GATE" }
                            ListElement { value: 13000; label: "SHIRA PEAK" }
                            ListElement { value: 15174; label: "LAVA TOWER" }
                            ListElement { value: 16893; label: "MAWENZI\nPEAK" }
                            ListElement { value: 19341; label: "SUMMIT" }
                        }

                        function getIndexModelForWorkout() {
                            switch (workoutProgram.programIndex) {
                                case WorkoutProgram.MOUNT_EVEREST_PROG:
                                    return mtEverestIndexPoints
                                case WorkoutProgram.MOUNT_RAINIER_PROG:
                                    return mtRainierIndexPoints
                                case WorkoutProgram.MOUNT_KILIMANJARO_PROG:
                                    return mtKilimanjaroIndexPoints
                            }
                            return null
                        }

                        Component.onCompleted: {
                            if (model.dataValue && model.dataValue.x !== undefined) {
                                elevationSlider.startValue = model.dataValue.x
                                elevationSlider.finishValue = model.dataValue.y
                                var pt = Qt.point(elevationSlider.startValue, elevationSlider.finishValue)
                                myListView.model.setProperty(index, "dataValue", pt);
                            }
                            completed = true
                        }

                        function handleValueChange() {
                            if (!completed) {
                                return
                            }

                            if (model.fieldType !== WorkoutProgram.ELEVATION_TYPE) {
                                return
                            }
                            var pt = Qt.point(elevationSlider.startValue, elevationSlider.finishValue)
                            myListView.model.setProperty(index, "dataValue", pt);
                        }

                        onStartValueChanged: {
                            handleValueChange()
                        }
                        onFinishValueChanged: {
                            handleValueChange()
                        }

                        startDisplayValue: options.metricUnits ? Globals.formatWithSeparator(Globals.feetToMeters(startValue).toFixed(0)) : Globals.formatWithSeparator(startValue)
                        finishDisplayValue: options.metricUnits ? Globals.formatWithSeparator(Globals.feetToMeters(finishValue).toFixed(0)) : Globals.formatWithSeparator(finishValue)

                        showValue: false
                        maxValue: model.maxValue
                        minValue: model.minValue
                        precision: model.precision
                        trackWidth: 1068 * main.screenScaleX
                        stepValue: options.metricUnits ? 82.021 : 100.0 // 25 meters / 100 feet
                        anchors.top: parent.top
                        anchors.topMargin: 60 * main.screenScaleY
                        anchors.left: parent.left
                        anchors.leftMargin: 45 * main.screenScaleX
                        width: 1068 * main.screenScaleX
                        height: 34 * main.screenScaleY
                        indexPoints: getIndexModelForWorkout()
                    }
                }

                Item {
                    // slider control box seen after expanding gender control
                    id: genderDetails
                    enabled: visible && opacity > 0
                    height: 155 * main.screenScaleY
                    width: 460 * main.screenScaleX
                    anchors.top: dataElement.top
                    anchors.topMargin: 85 * main.screenScaleY
                    anchors.left: dataElement.left
                    anchors.leftMargin: 50 * main.screenScaleX
                    opacity: 0

                    Image {
                        anchors.horizontalCenter: parent.horizontalCenter
                        anchors.verticalCenter: parent.verticalCenter
                        source: main.graphicsPath + "setup/Setup_Large_Up_Step2.png"
                    }

                    SliderControl {
                        id: genderSlider

                        onValueChanged: {
                            SetupJS.genderSliderValueChanged(model, value, index)
                        }
                        showValue: false
                        maxValue: 1 // FEMALE
                        minValue: 0 // MALE
                        value: model.dataValue
                        precision: 0
                        showButtons: false
                        anchors.top: parent.top
                        anchors.topMargin: 75 * main.screenScaleY
                        anchors.horizontalCenter: parent.horizontalCenter
                        anchors.horizontalCenterOffset: -20 * main.screenScaleX
                        width: 300 * main.screenScaleX - femaleText.width - maleText.width
                        height: 34 * main.screenScaleY

                        Text {
                            id: maleText
                            width:paintedWidth
                            anchors { right: parent.left ; rightMargin: 40 * main.screenScaleX; verticalCenter: parent.verticalCenter }
                            text: qsTr("MALE") + application.emptyString
                            MouseArea {
                                anchors.fill: parent ; onClicked: genderSlider.value = 0
                            }
                        }

                        Text {
                            id: femaleText
                            width:paintedWidth
                            text: qsTr("FEMALE") + application.emptyString
                            anchors { left: parent.right ; leftMargin: 40 * main.screenScaleX; verticalCenter : parent.verticalCenter  }
                            MouseArea {
                                anchors.fill: parent ; onClicked: genderSlider.value = 1
                            }
                        }
                    }
                }

                Item {
                    // distance selection drop-down with preset distance buttons
                    id: expandedDetails1
                    enabled: visible && opacity > 0
                    height: 155 * main.screenScaleY
                    width: 460 * main.screenScaleX
                    anchors.top: dataElement.top
                    anchors.topMargin: 85 * main.screenScaleY
                    anchors.left: dataElement.left
                    anchors.leftMargin: 40 * main.screenScaleX
                    opacity: dataElement.expandedDetails1Opacity

                    Image {
                        id: dataIndicatorBackground2
                        anchors.horizontalCenter: parent.horizontalCenter
                        anchors.verticalCenter: parent.verticalCenter
                        source: main.graphicsPath + "setup/Setup_Large_Up_Step2.png"
                    }

                    ImageButton {
                        id: fiveKButton
                        property bool valueMatches: (dataValue == 5) && (units == "K")

                        anchors.left: dataIndicatorBackground2.left
                        anchors.leftMargin: 50 * main.screenScaleX
                        anchors.verticalCenter: dataIndicatorBackground2.verticalCenter
                        anchors.verticalCenterOffset: 10 * main.screenScaleY
                        downimage: main.graphicsPath + "205/SM_Button_Down.png"
                        upimage: valueMatches ? main.graphicsPath + "205/SM_Button_Down.png": main.graphicsPath + "205/SM_Button_Up.png"
                        fontsize: 14 * main.screenScaleY
                        label_color: "black"
                        label: qsTr("5K") + application.emptyString
                        onClicked: {
                            myListView.model.setProperty(index, "dataValue", 5);
                            myListView.model.setProperty(index, "units", "K");
                            dataItemChanged()
                        }
                    }

                    ImageButton {
                        id: tenKButton
                        property bool valueMatches: (dataValue == 10) && (units == "K")
                        enabled: workoutProgram.programIndex !== WorkoutProgram.STRIDE_DISTANCE_PROG

                        anchors.left: fiveKButton.right
                        anchors.leftMargin: 25 * main.screenScaleX
                        anchors.verticalCenter: dataIndicatorBackground2.verticalCenter
                        anchors.verticalCenterOffset: 10 * main.screenScaleY
                        downimage: main.graphicsPath + "205/SM_Button_Down.png"
                        upimage: valueMatches ? main.graphicsPath + "205/SM_Button_Down.png": main.graphicsPath + "205/SM_Button_Up.png"
                        fontsize: 14 * main.screenScaleY
                        label_color: "black"
                        label: qsTr("10K") + application.emptyString
                        onClicked: {
                            myListView.model.setProperty(index, "dataValue", 10);
                            myListView.model.setProperty(index, "units", "K");
                            dataItemChanged()
                        }
                    }

                    ImageButton {
                        property bool valueMatches: (dataValue == 2) && (units == "MI")

                        id: twoMileButton
                        enabled: workoutProgram.programIndex !== WorkoutProgram.STRIDE_DISTANCE_PROG
                        anchors.left: tenKButton.right
                        anchors.leftMargin: 25 * main.screenScaleX
                        anchors.verticalCenter: dataIndicatorBackground2.verticalCenter
                        anchors.verticalCenterOffset: 10 * main.screenScaleY
                        downimage: main.graphicsPath + "205/SM_Button_Down.png"
                        upimage: valueMatches ? main.graphicsPath + "205/SM_Button_Down.png": main.graphicsPath + "205/SM_Button_Up.png"
                        fontsize: 14 * main.screenScaleY
                        label_color: "black"
                        label: qsTr("2MI") + application.emptyString
                        onClicked: {
                            myListView.model.setProperty(index, "dataValue", 2);
                            myListView.model.setProperty(index, "units", "MI");
                            dataItemChanged()
                        }
                    }

                    ImageButton {
                        property bool valueMatches: (dataValue == 4) && (units == "MI")

                        id: fourMileButton
                        enabled: workoutProgram.programIndex !== WorkoutProgram.STRIDE_DISTANCE_PROG
                        anchors.left: twoMileButton.right
                        anchors.leftMargin: 25 * main.screenScaleX
                        anchors.verticalCenter: dataIndicatorBackground2.verticalCenter
                        anchors.verticalCenterOffset: 10 * main.screenScaleY
                        downimage: main.graphicsPath + "205/SM_Button_Down.png"
                        upimage: valueMatches ? main.graphicsPath + "205/SM_Button_Down.png": main.graphicsPath + "205/SM_Button_Up.png"
                        fontsize: 14 * main.screenScaleY
                        label_color: "black"
                        label: qsTr("4MI") + application.emptyString
                        onClicked: {
                            myListView.model.setProperty(index, "dataValue", 4);
                            myListView.model.setProperty(index, "units", "MI");
                            dataItemChanged()
                        }
                    }
                }

                Item {
                    // heart rate setting drop down sliders
                    id: expandedDetails2
                    enabled: visible && opacity > 0
                    height: 155 * main.screenScaleY
                    width: 460 * main.screenScaleX
                    anchors.top: dataElement.top
                    anchors.topMargin: 85 * main.screenScaleY
                    anchors.left: dataElement.left
                    anchors.leftMargin: 40 * main.screenScaleX
                    opacity: dataElement.expandedDetails2Opacity

                    Image {
                        id: dataIndicatorBackground3
                        anchors.horizontalCenter: parent.horizontalCenter
                        anchors.verticalCenter: parent.verticalCenter
                        source: main.graphicsPath + "setup/Setup_Large_Up_Step2.png"
                    }

                    SliderControl {
                        id: sliderObject2

                        Component.onCompleted: {
                            if (model.fieldType === WorkoutProgram.ELEVATION_TYPE) {
                                // Avoid warning about assigning QPointF to double
                                return
                            }

                            value = model.dataValue
                        }

                        onValueChanged: {
                            SetupJS.sliderObject2ValueChanged(model, value, index)
                        }
                        showValue: false
                        maxValue: model.maxValue
                        minValue: model.minValue
                        value: model.dataValue
                        anchors.top: parent.top
                        anchors.topMargin: 75 * main.screenScaleY
                        anchors.left: parent.left
                        anchors.leftMargin: 45 * main.screenScaleX
                        width: 350 * main.screenScaleX
                        height: 34 * main.screenScaleY
                    }
                }

                Item {
                    // drop down slider for setting number of segments for custom profile programs
                    id: segmentDetails
                    enabled: visible && opacity > 0
                    height: 155 * main.screenScaleY
                    width: 460 * main.screenScaleX
                    anchors.top: dataElement.top
                    anchors.topMargin: 85 * main.screenScaleY
                    anchors.left: dataElement.left
                    anchors.leftMargin: 50 * main.screenScaleX
                    opacity: dataElement.segmentDetailsOpacity

                    Image {
                        id: dataIndicatorBackground5
                        anchors.top: sliderObject3.top
                        anchors.topMargin: 15 * main.screenScaleY
                        anchors.left: sliderObject3.left
                        anchors.leftMargin: 15 * main.screenScaleX
                        source: main.graphicsPath + "setup/Setup_Large_Up_Step3.png"
                    }

                    Text {
                        id: calculatedSegTime1
                        anchors.verticalCenter: dataIndicatorBackground5.verticalCenter
                        anchors.left: dataIndicatorBackground5.left
                        anchors.leftMargin: 30 * main.screenScaleX
                        text: qsTr("SEGMENT TIME") + application.emptyString
                        font.pointSize: 18 * main.screenScaleY
                        font.bold: true
                        smooth: true
                        color: "black"
                        z: 2
                    }

                    Text {
                        id: calculatedSegTimeValue1
                        anchors.verticalCenter: dataIndicatorBackground5.verticalCenter
                        anchors.right: dataIndicatorBackground5.right
                        anchors.rightMargin: 40 * main.screenScaleX
                        text: Format.formatTimeString(mainProgItem.segTime);
                        font.pointSize: 32 * main.screenScaleY
                        font.bold: true
                        smooth: true
                        color: "black"
                        z: 2
                    }

                    Text {
                        id: calculatedSegTimeNote
                        anchors.verticalCenter: dataIndicatorBackground5.verticalCenter
                        anchors.verticalCenterOffset: 30 * main.screenScaleY
                        anchors.right: dataIndicatorBackground5.right
                        anchors.rightMargin: 40 * main.screenScaleX
                        text: qsTr("(MINIMUM SEGMENT TIME IS 1:00)") + application.emptyString
                        font.pointSize: 12 * main.screenScaleY
                        font.bold: true
                        smooth: true
                        color: "black"
                        z: 2
                    }

                    Image {
                        id: dataIndicatorBackground4
                        anchors.horizontalCenter: parent.horizontalCenter
                        anchors.verticalCenter: parent.verticalCenter
                        source: main.graphicsPath + "setup/Setup_Large_Up_Step2.png"
                    }

                    SliderControl {
                        id: sliderObject3

                        onValueChanged: {
                            SetupJS.sliderObject3ValueChanged(model, value, index)
                        }
                        showValue: false
                        minValue: model.minValue
                        maxValue: model.maxValue
                        value: model.dataValue
                        precision: model.precision
                        stepValue: {
                            if (model.precision == 0) {
                                return 1;
                            }
                            else if (model.precision == 1) {
                                return 0.1;
                            }
                        }

                        anchors.top: parent.top
                        anchors.topMargin: 75 * main.screenScaleY
                        anchors.left: parent.left
                        anchors.leftMargin: 45 * main.screenScaleX
                        width: 350 * main.screenScaleX
                        height: 34 * main.screenScaleY
                    }
                }

                Item {
                    id: levelDetails
                    enabled: visible && opacity > 0
                    height: 155 * main.screenScaleY
                    width: 460 * main.screenScaleX
                    anchors.top: dataElement.top
                    anchors.topMargin: 175 * main.screenScaleY
                    anchors.left: dataElement.left
                    anchors.leftMargin: 110 * main.screenScaleX
                    z: details.z - 1
                    opacity: dataElement.levelDetailsOpacity

                    Text {
                        id: levelSubLabel
                        anchors.verticalCenter: dataIndicatorBackground7.verticalCenter
                        anchors.left: dataIndicatorBackground7.left
                        anchors.leftMargin: 30 * main.screenScaleX
                        text: SetupJS.levelDetailsGetLabel() + application.emptyString
                        font.pointSize: 18 * main.screenScaleY
                        font.bold: true
                        smooth: true
                        color: "black"
                        z: 2
                    }

                    Text {
                        id: levelSubValue
                        anchors.verticalCenter: dataIndicatorBackground7.verticalCenter
                        anchors.right: levelSubUnits.left
                        anchors.rightMargin: 1 * main.screenScaleX
                        text: SetupJS.levelDetailsGetSubValue(details.value)
                        font.pointSize: 32 * main.screenScaleY
                        font.bold: true
                        smooth: true
                        color: "black"
                        z: 2
                    }

                    Text {
                        id: levelSubUnits
                        anchors.bottomMargin: height/2
                        anchors.bottom: levelSubValue.bottom
                        anchors.right: dataIndicatorBackground7.right
                        anchors.rightMargin: 40 * main.screenScaleX
                        text: SetupJS.levelDetailsGetUnits() + application.emptyString
                        font.pointSize: 12 * main.screenScaleY
                        font.bold: true
                        smooth: true
                        color: "black"
                        z: 2
                    }

                    Image {
                        id: dataIndicatorBackground7
                        anchors.horizontalCenter: parent.horizontalCenter
                        anchors.verticalCenter: parent.verticalCenter
                        source: main.graphicsPath + "setup/Setup_Large_Up_Step3.png"
                    }

                }

                states: [
                    State {
                        name: "Details"
                        PropertyChanges { target: background; height: 160 * main.screenScaleY; width: 455 * main.screenScaleX; anchors.leftMargin: -1 * main.screenScaleX; anchors.topMargin: 1 * main.screenScaleY; source: main.graphicsPath + "setup/Setup_Large_Up_Step1.png" }
                        PropertyChanges { target: itemValue; opacity: 0.0}
                        PropertyChanges { target: itemUnits; visible: false}
                        PropertyChanges { target: itemValueDetails; visible: true }
                        PropertyChanges { target: itemUnitsDetails; visible: true }
                        PropertyChanges { target: dataElement; height: 225 * main.screenScaleY } // Fill the entire list area with the detailed view
                        PropertyChanges { target: dataElement; detailsOpacity: 1} // Make details visible
                        PropertyChanges { target: dataElement; elevationDetailsOpacity: 0 }
                    },
                    State {
                        name: "ElevationDetails"
                        PropertyChanges { target: background; height: 160 * main.screenScaleY; width: 455 * main.screenScaleX; anchors.leftMargin: -1 * main.screenScaleX; anchors.topMargin: 1 * main.screenScaleY; source: main.graphicsPath + "setup/Setup_Large_Up_Step1.png" }
                        PropertyChanges { target: itemValue; opacity: 0.0}
                        PropertyChanges { target: itemUnits; visible: false}
                        PropertyChanges { target: itemValueDetails; visible: true }
                        PropertyChanges { target: itemUnitsDetails; visible: true }
                        PropertyChanges { target: dataElement; height: 225 * main.screenScaleY } // Fill the entire list area with the detailed view
                        PropertyChanges { target: dataElement; elevationDetailsOpacity: 1 }
                    },
                    State {
                        name: "LevelDetails"
                        PropertyChanges { target: background; height: 160 * main.screenScaleY; width: 455 * main.screenScaleX; anchors.leftMargin: -1 * main.screenScaleX; anchors.topMargin: 1 * main.screenScaleY; source: main.graphicsPath + "setup/Setup_Large_Up_Step1.png" }
                        PropertyChanges { target: itemValue; opacity: 0.0}
                        PropertyChanges { target: itemUnits; visible: false}
                        PropertyChanges { target: itemValueDetails; visible: true }
                        PropertyChanges { target: itemUnitsDetails; visible: true }
                        PropertyChanges { target: dataElement; height: 315 * main.screenScaleY } // Fill the entire list area with the detailed view
                        PropertyChanges { target: dataElement; detailsOpacity: 1} // Make details visible
                        PropertyChanges { target: dataElement; levelDetailsOpacity: 1} // Make details visible
                        PropertyChanges { target: dataElement; elevationDetailsOpacity: 0 }
                    },
                    State {
                        name: "ExpandedDetails1"
                        PropertyChanges { target: background; height: 160 * main.screenScaleY; width: 455 * main.screenScaleX; anchors.leftMargin: -1 * main.screenScaleX; anchors.topMargin: 1 * main.screenScaleY; source: main.graphicsPath + "setup/Setup_Large_Up_Step1.png" }
                        PropertyChanges { target: itemValue; opacity: 0.0}
                        PropertyChanges { target: itemUnits; visible: false}
                        PropertyChanges { target: itemValueDetails; visible: true }
                        PropertyChanges { target: itemUnitsDetails; visible: true }
                        PropertyChanges { target: dataElement; height: 225 * main.screenScaleY } // Fill the entire list area with the detailed view
                        PropertyChanges { target: dataElement; expandedDetails1Opacity: 1} // Make details visible
                        PropertyChanges { target: dataElement; elevationDetailsOpacity: 0 }
                    },
                    State {
                        name: "Expanded2"
                        AnchorChanges { target: itemUnits
                            anchors.top: parent.top
                            anchors.right: parent.right
                        }
                        PropertyChanges { target: itemUnits;
                            anchors.topMargin: ( application.currentLanguage == "ar" ? 60 : 90 ) * main.screenScaleY
                            anchors.rightMargin: 200 * main.screenScaleX
                            width: application.currentLanguage == "ar" ? 120 : implicitWidth
                            font.pointSize: application.currentLanguage == "ar" ? 12 * main.screenScaleY : 18 * main.screenScaleY
                        }
                        PropertyChanges { target: itemValue; anchors.bottomMargin: -6 * main.screenScaleY; font.pointSize: 32 * main.screenScaleY }
                        PropertyChanges { target: hrDividerText; visible: true }
                        PropertyChanges { target: hrPercentageValue; visible: true }
                        PropertyChanges { target: hrPercentageUnits; visible: true }
                    },
                    State {
                        name: "Expanded3"
                        AnchorChanges { target: itemUnits; anchors.top: parent.top; anchors.right: parent.right }
                        PropertyChanges { target: itemUnits; anchors.topMargin: 90 * main.screenScaleY; anchors.rightMargin: 20 * main.screenScaleX; font.pointSize: 18 * main.screenScaleY }
                        PropertyChanges { target: itemValue; anchors.bottomMargin: -6 * main.screenScaleY; font.pointSize: 32 * main.screenScaleY }
                    },
                    State {
                        name: "ExpandedDetails2"
                        PropertyChanges { target: background; height: 160 * main.screenScaleY; width: 455 * main.screenScaleX; anchors.leftMargin: -1 * main.screenScaleX; anchors.topMargin: 1 * main.screenScaleY; source: main.graphicsPath + "setup/Setup_Large_Up_Step1.png" }
                        PropertyChanges { target: itemValue; opacity: 0.0}
                        PropertyChanges { target: itemUnits; visible: false}
                        AnchorChanges { target: itemUnitsDetails
                            anchors.top: parent.top
                            anchors.right: parent.right
                        }
                        PropertyChanges { target: itemUnitsDetails
                            anchors.topMargin: ( application.currentLanguage == "ar" ? 81 : 90 ) * main.screenScaleY
                            anchors.rightMargin: 200 * main.screenScaleX
                            font.pointSize: application.currentLanguage == "ar" ? 12 * main.screenScaleY : 18 * main.screenScaleY
                            width: application.currentLanguage == "ar" ? 140 : implicitWidth
                        }
                        PropertyChanges { target: itemValueDetails; anchors.bottomMargin: -6 * main.screenScaleY; font.pointSize: 32 * main.screenScaleY }
                        PropertyChanges { target: itemValueDetails; visible: true }
                        PropertyChanges { target: itemUnitsDetails; visible: true }
                        PropertyChanges { target: hrDividerText; visible: true }
                        PropertyChanges { target: hrPercentageValue; visible: true }
                        PropertyChanges { target: hrPercentageUnits; visible: true }
                        PropertyChanges { target: dataElement; height: 225 * main.screenScaleY } // Fill the entire list area with the detailed view
                        PropertyChanges { target: dataElement; expandedDetails2Opacity: 1} // Make details visible
                        PropertyChanges { target: dataElement; elevationDetailsOpacity: 0 }
                    },
                    State {
                        name: "SegmentDetails"
                        PropertyChanges { target: background; height: 160 * main.screenScaleY; width: 455 * main.screenScaleX; anchors.leftMargin: -1 * main.screenScaleX; anchors.topMargin: 1 * main.screenScaleY; source: main.graphicsPath + "setup/Setup_Large_Up_Step1.png" }
                        PropertyChanges { target: itemValue; opacity: 0.0}
                        PropertyChanges { target: itemUnits; visible: false}
                        PropertyChanges { target: itemValueDetails; visible: true }
                        PropertyChanges { target: itemUnitsDetails; visible: true }
                        PropertyChanges { target: dataElement; height: 315 * main.screenScaleY } // Fill the entire list area with the detailed view
                        PropertyChanges { target: dataElement; segmentDetailsOpacity: 1} // Make details visible
                        PropertyChanges { target: dataElement; elevationDetailsOpacity: 0 }
                    },
                    State {
                        name: "GenderDetails"
                        PropertyChanges { target: background; height: 160 * main.screenScaleY; width: 455 * main.screenScaleX; anchors.leftMargin: -1 * main.screenScaleX; anchors.topMargin: 1 * main.screenScaleY; source: main.graphicsPath + "setup/Setup_Large_Up_Step1.png" }
                        PropertyChanges { target: itemValue; opacity: 0.0}
                        PropertyChanges { target: itemUnits; visible: false}
                        PropertyChanges { target: itemValueDetails; visible: true }
                        PropertyChanges { target: itemUnitsDetails; visible: true }
                        PropertyChanges { target: dataElement; height: 225 * main.screenScaleY } // Fill the entire list area with the detailed view
                        PropertyChanges { target: genderDetails; opacity: 1} // Make details visible
                        PropertyChanges { target: dataElement; elevationDetailsOpacity: 0 }
                    }

                ]

                transitions: [
                    Transition {
                        to: "Details"
                        reversible: true
                    },
                    Transition {
                        to: "ExpandedDetails1"
                        reversible: true
                    },
                    Transition {
                        to: "ExpandedDetails2"
                        reversible: true
                    },
                    Transition {
                        to: "SegmentDetails"
                        reversible: true
                    }
                ]
            }
        }
    }

    Rectangle { id: cautionMessage

        property variant item: myListView.model ? myListView.model.get(selectedIndex) : undefined
        property bool showInclineMessage: item !== undefined && isTreadmill && (item.fieldType === WorkoutProgram.INCLINE_TYPE || item.fieldType === WorkoutProgram.MAX_INCLINE_TYPE || (item.fieldType === WorkoutProgram.LEVEL_TYPE && workoutProgram.programIndex !== WorkoutProgram.TREAD_LEG_SHAPER_PROG))
        property bool showWorkloadMessage: item !== undefined && !isTreadmill && (item.fieldType === WorkoutProgram.LEVEL_TYPE || item.fieldType === WorkoutProgram.WORKLOAD_TYPE)
        property bool showSpeedMessage: item !== undefined && (SetupJS.isSpeedType(item.fieldType) || (item.fieldType === WorkoutProgram.LEVEL_TYPE && workoutProgram.programIndex === WorkoutProgram.TREAD_LEG_SHAPER_PROG))

        visible: showInclineMessage || showWorkloadMessage || showSpeedMessage

        anchors.bottom: parent.bottom
        anchors.bottomMargin: 10 * main.screenScaleY
        anchors.left: parent.left
        anchors.leftMargin: 10 * main.screenScaleX
        width: childrenRect.width
        height:childrenRect.height
        color: "#e8e8e8"

        Image{
            id: cautionSymbol
            source: main.graphicsPath + "Icon_Warning.png"
        }

        Text{
            id: cautionTitle
            text: qsTr("CAUTION") + application.emptyString
            font.bold: true
            font.pixelSize: 22 * main.screenScaleY
            anchors.left: cautionSymbol.right
            anchors.leftMargin: 5 * main.screenScaleX
            anchors.verticalCenter: cautionSymbol.verticalCenter
            anchors.verticalCenterOffset: 0
        }

        Text{
            id: cautionText
            width: 450 * main.screenScaleX
            wrapMode: Text.WordWrap
            text: {
                if (cautionMessage.showInclineMessage)
                    return qsTr("Use caution when selecting incline level. Injury can result if not prepared for incline levels beyond your physical limits.") + application.emptyString
                else if (cautionMessage.showWorkloadMessage)
                    return qsTr("Use caution when selecting level. Injury can result if not prepared for workload resistance beyond your physical limits.") + application.emptyString
                else if (cautionMessage.showSpeedMessage)
                    return qsTr("Use caution when selecting speed. Injury can result if not prepared for speeds beyond your physical limits.") + application.emptyString
                else
                    return ""
            }
            font.pixelSize: 16 * main.screenScaleY
            anchors.left: cautionSymbol.right
            anchors.leftMargin: 5 * main.screenScaleX
            anchors.top: cautionTitle.bottom
            anchors.topMargin: 5 * main.screenScaleY
        }
    }

}

