/** @file main.qml
    @brief Manage main display and transitions to workout menus and run screens.
*/

import QtQuick 2.4
import WorkoutManager 1.0
import StorageMgr 1.0
import StatisticManager 1.0
import ErrorManager 1.0
import WorkoutRunData 1.0
import Process 0.1
import AMixer 1.0
import Netpulse 0.1
import Bluetooth 1.0
import HeartRateManager 1.0
import RecoveryManager 1.0
import SapManager 1.0
import Components 1.0
import FileIO 1.0
import "LanguageMenu"
import "Diagnostics"
import "MediaBrowser"
import "tft_workout_menus.js" as TftMenus
import "workout_menus.js" as TouchMenus
import "workout_menus.js" as Menus
import "string_format.js" as StrFormat
import "Television"
import "Keypad"

import VLCVideo 1.0

Item {
    id: main

    property bool netpulseAvailable: options.netpulseEnabled && netpulse.connected
    property variant topmenu
    property variant allWorkoutsMenu

    property bool _dialogShowing: false
    property bool blockKeys: restartRequired || dialogManager.anyAreBlockingKeys || popupManager.estopShown || logoImage.blinking || main.quickStartLoading

    property bool showStimulusArea: options.showStimulusArea

    property int volume: 5
    property real timeFactor: 1.0
    property string serialNumber: application.serialNumber
    property string productVersion: application.productVersion
    property bool isTreadmill: machineinfo.productTypeString == "Treadmill"
    property bool isTft: application.isTft
    property bool isTouch: !application.isTft
    property bool rightToLeft: Qt.application.layoutDirection === Qt.RightToLeft
    property string channelSelected: ""
    property bool restartRequired: false
    property bool rebootRequired: false
    property bool firmwareUpdateRequired: false
    property bool _hideLanguageButton: false
    property int screenShotCount: 0
    property string consoleType: application.consoleType
    property double screenScaleX: application.screenScaleX
    property double screenScaleY: application.screenScaleY
    property string graphicsPath: application.graphicsPath
    property bool _debugKeys: true
    property bool erpIsSupported: true
    property bool quickStartLoading: false
    property bool isCSA: machineinfo.modelString === "TS1000" || machineinfo.modelString === "XS1000"
    property bool isLateralTrainer: machineinfo.modelString === "TL1000" || machineinfo.modelString === "XL1000"
    property bool isAlpineRunner: machineinfo.modelString === "TI1000"

    property variant home_screen: home_screen_loader.item
    property variant workout_menu: workout_menu_loader.item
    property string lowerSystem: ""
    property bool canStartWorkout: true

    width: 1366 * main.screenScaleX
    height: 768 * main.screenScaleY
    focus: state == "HomeState" || state == "LoginState" || state == "RegisterState"
    signal reset()

    Component.onCompleted: {
        QT_TRANSLATE_NOOP("QApplication", "QT_LAYOUT_DIRECTION")
        // These additional contexts must be added to enable right-to-left layout for a language
        // Previously, only QApplication was needed. This is a workaround for QTBUG-43447 in Qt5.x
        QT_TRANSLATE_NOOP("QCoreApplication", "QT_LAYOUT_DIRECTION")
        QT_TRANSLATE_NOOP("QGuiApplication", "QT_LAYOUT_DIRECTION")
        // start on the home screen
        print("***** Application Start v" + application.appVersion + " *****") ;
        state = "HomeState"
        print("Getting Bios Version...")
        getBiosVerProcess.start()
        var lang = options.defaultLanguage
        console.log("language default: " +lang)
        application.selectLanguage(lang)
        selectWorkoutMenu()
        buzzer.buzzerEnabled = !options.disableBuzzer

        /* On start of application, set simulationMode and ignoreFaults to false */
        if (!options.manufacturingTestModeEnabled) {
            options.simulationMode = false
            options.ignoreFaults = false
        }
        else {
            options.ignoreFaults = true
        }

        if (application.crashRecovery) {
            logUploadPrompt.show()
            logUploadPrompt.start()
        }

      
        if(options.triggerWorkoutAutoRestart) {
            // Automatically start Quickstart for TV testing
            console.log("===================================> Quickstart starting in " + autostartTimer.interval + " ms")
            autostartTimer.start()
        }
    }



    Timer{
        id: autostartTimer
        // if root-hub is being reset, wait 3 minutes else 50 seconds
        interval: tv.usbReseting === true ? 180000 : 50000
        running: false
        repeat: false
        onTriggered:{
            if (main.rebootRequired)
                return

            if(!tv.usbReseting && nucleus.systemReady) {
                console.log("automatically restarting workout ")
                tv.channelUp() // Turn tv on to first channel so view automaticaly changes to TV view
                quickStart()
            } else {
                autostartTimer.restart()
            }
        }
    }

    Timer{
        id: autostopTimer
        interval: 1800000  //30 minutes
        running: false
        repeat: false
        onTriggered:{
            workoutScreen.stopWorkout();
            autostartTimer.start()
        }
    }


    Component.onDestruction: {
        urlKeypad.process.terminate()
        inputKeypad.process.terminate()
        if (bluetoothAdapter.connected)
            bluetoothAdapter.disconnect()
        if (bluetoothAdapter.connectedHeadset)
            bluetoothAdapter.disconnectHeadset()
        console.log("Main: Component destroyed")
    }

    Binding { target: application; property: "swipeStartX"; value: 0; when: main.state != "RunState" }
    Binding { target: application; property: "swipeEndX"; value: main.width; when: main.state != "RunState" }
    Binding { target: application; property: "swipeStartY"; value: 0; when: main.state != "RunState" }
    Binding { target: application; property: "swipeEndY"; value: main.height; when: main.state != "RunState" }

    Process {
        id: getBiosVerProcess
        Component.onCompleted: start()
        program: "sudo"
        arguments: ["cat", "/sys/class/dmi/id/bios_version"]
        onReadyReadStandardOutput: {
            var version = String(readAllStandardOutput())
            version = version.trim()
            print("Bios version is " + version)
        }
    }

    NetworkConnections {
        id: netConnections
        onIsRefreshingChanged: {
            if (!isRefreshing) {
                console.log("NetworkConnections: Finished refreshing connections")
                netConnections.printStatus()
            } else {
                console.log("NetworkConnections: Started refreshing connections")
            }
        }
    }

    Connections {
        target: nucleus
        onNucleusConnectionChanged: {
            if(connection == true) {
                nucleus.startInitializationProcess()
            }
        }
        onInitializationProcessFinished: {
            if(result == true) {
                selectWorkoutMenu()
                console.log("Nucleus: Diagnostics = " + options.loggingEnabled)
                nucleus.setDiagnostics(options.loggingEnabled)
                nucleus.cabSetEnable(options.cabEnabled)

                // Retrieve Nucleus error logs
                console.log("Retrieving Nucleus fault log")
                errorManager.retrieveFaultLogs()

                nucleusfirmwareloader.findHexFile()
            }
        }

        onSystemReadyChanged: {
            console.log("Nucleus: System Ready = " + nucleus.systemReady)
            if(nucleus.systemReady === true) {
                canStartWorkout = true                
            } else {
                canStartWorkout = false
            }
        }

    }

    Connections{
        target: tv
        onTunerTypeChanged: {
            switch(tv.tunerType){
            case "ATSC":
                erpIsSupported = true
                break;
            default:
                erpIsSupported = true
                break;
            }
        }
    }

    function debugKeys(keyname, objectname, notes) {
        if (main._debugKeys)
            console.log("KEYPRESS: ",  keyname, objectname, notes)
    }

    function erpSleep() {
        if (lowerSystem === "NUCLEUS") {
            nucleus.erpSleep()
        }
    }

    DialogManager {
        id: dialogManager
    }

    Process { id: syncDisk
        program: "/bin/sync"
        block: true
        blockTimeout: 1000 * 60 * 60; // 1 hour
        onFinished: {
            syncNoBlockTimer.restart()
        }
    }

    Timer {
        id: syncNoBlockTimer
        interval: 1000
        repeat: false
        running: false
        onTriggered: {
            syncNoBlock.start()
        }
    }
    Process {
        id: syncNoBlock
        program: "/bin/sync"
    }

    MessagePopup {
        id: beltStoppingPopup
        blockKeys: false
        isTransparent: true
        property bool showPopup: main.state === "HomeState" && popupManager.estopShown === false && isTreadmill && (machinecontrol.isBeltMoving === true || machinecontrol.isInclineMoving === true || (lowerSystem === "NUCLEUS" && nucleus.systemReady === false))
        onShowPopupChanged: {
            if (showPopup)
                show()
            else
                hide()
        }
        message: qsTr("Please wait...") + application.emptyString
        Component.onCompleted: hide()
    }

    MessagePopup {
        id: nucleusNotReadyPopup
        blockKeys: false
        isTransparent: true

        /**
         * @brief type:bool Property that determines if the pop-up window is to be displayed.
         */
        property bool showPopup: main.state === "HomeState" && !isTreadmill && lowerSystem === "NUCLEUS" && nucleus.systemReady === false

        onShowPopupChanged: {
            if (showPopup) {
                show();
            }
            else {
                hide();
            }
        }
        message: qsTr("Nucleus Not Ready...") + application.emptyString
        Component.onCompleted: hide()
    }

    MessagePopup {
        id: blockUseMessage
        property bool showPopup: main.state !== "DiagnosticState" && main.state !== "SetupWizardState" && (restartRequired || home_screen.buttonsEnabled === false)
        onShowPopupChanged: {
            if (showPopup) {
                logger.logAlways("Show BlockUseMessage: " + blockUseMessage.message)
                show();
            }
            else {
                logger.logAlways("BlockUseMessage hidden")
                hide();
            }
        }
        message: {
            if (main.restartRequired) {
                return (application.crashRecovery ? qsTr("Power Cycle Required") : tv.usbResetFailed ? qsTr("TV Error Detected.") + "\n" + qsTr("Power Cycle Required") : qsTr("Power Cycle Required to Complete Update")) + application.emptyString
            }else if (!machineinfo.isReady)
                return qsTr("Starting up...") + application.emptyString
            else if (!home_screen.buttonsEnabled) {
                if (machineinfo.productTypeString === "" && options.ignoreFaults == false) {
                    return qsTr("Console Configuration Required.") + application.emptyString
                } else if (machineinfo.productTypeString === "Treadmill" && machinecontrol.isCalibrated == false && options.ignoreFaults == false){
                    return qsTr("Treadmill Calibration Required.") + application.emptyString
                } else if (machineinfo.productTypeString === "" && options.ignoreFaults == true){
                    return qsTr("Product Configuration Required.") + application.emptyString
                } else {
                    return qsTr("Complete Configuration to Enable Buttons") + application.emptyString
                }
            } else {
                return qsTr("Complete Configuration to Enable Buttons") + application.emptyString
            }
        }
        Component.onCompleted: if (showPopup) show(); else hide()
        isTransparent: true

        ImageButton {
            id: rebootButton
            property bool showButton: true

            x: blockUseMessage.xOffset + 350 * main.screenScaleX
            y: blockUseMessage.yOffset + 325 * main.screenScaleY
            z: blockUseMessage.z + 1
            visible: main.restartRequired && rebootButton.showButton

            label: qsTr("Reboot") + application.emptyString
            label_color: "black"
            label_y_offset: -2 * main.screenScaleY
            label_x_offset: -2 * main.screenScaleX
            fontsize: 14 * main.screenScaleY
            scale: 1.5
            upimage: main.graphicsPath + "205/Lrg_Button_Up.png"
            downimage: main.graphicsPath + "205/Lrg_Button_Down.png"

            onClicked: {
                main.startReboot()
                rebootButton.showButton = false
                blockUseMessage.message = qsTr("Rebooting Now!")
            }
        }
    }

    MiscStats {
        id: miscStats
    }

    WorkoutStats {
        id: workoutStats
    }

    UserData { id: userData
        loggedIn: false
    }

    DataExport {
        id: dataExport
        workoutProgram: workoutProgram
        workoutData: aWorkoutRunData
        userData: userData
        profile1: workoutProgram.profile1
        profile2: workoutProgram.profile2
        productModel: machineinfo.modelString
        machineID: netpulse.machineID
        serialNumber: main.serialNumber
        runMiles: statisticManager.totalDistance
        runHours: statisticManager.runHours / 3600  // stats manager keeps "runHours" in seconds
        modelID: 0 //configurationData.csafeID
        version: application.appVersion
        Component.onCompleted: print("ip address:" + ipAddress)
    }

    Netpulse { id: netpulse
        userLogin: userData
        frozen: main._dialogShowing || workoutScreen.stopPrompt.visible || dialogManager.anyAreShown
    }

    NetpulseLogin {
        id: loginWindow

        onTimeout: {
            if(main.state === "LoginState" || main.state == "RegisterState") {
                netpulse.hideAll()
                main.state="HomeState";
            }
        }
        onLoggedIn: {
            if(main.state === "LoginState" || main.state == "RegisterState") {
                main.state="HomeState";
            }
        }
        anchors.fill: parent
    }

    Connections{
        target: tv
        onTunerTypeChanged: {
            switch(tv.tunerType){
            case "ATSC":
                erpIsSupported = true
                break;
            default:
                erpIsSupported = true
                break;
            }
        }
    }

    NetpulseRegistration {
        id: registerWindow

        onTimeout: {
            netpulse.hideAll()
            main.state="HomeState";
        }
        onLoggedIn: {
            if(main.state === "LoginState" || main.state == "RegisterState") {
                main.state="HomeState";
            }
        }
        anchors.fill: parent
    }

    NetpulseInstall {
        id: netpulseInstall

        anchors.fill: parent
    }

/*
    NetpulseSummary {
        id: netpulseSummary

        anchors.fill: parent
    }
*/
    NetpulseWatch {
        id: netpulseWatch

        leftSideDisplay: workoutScreen.sideDataDisplay.leftDisplay
        rightSideDisplay: workoutScreen.sideDataDisplay.rightDisplay

        onStopButtonPressed: {
            console.log("NetpulseWatch: Stop button pressed")
            workoutScreen.stopButtonPressedHandler()
        }

        z: 99
    }

    // expire Netpulse Login after 30 seconds with no activity on homescreen
    Timer {
        id: netpulseLoginTimer

        running: (main.state == "HomeState") && netpulse.loggedIn
        interval: 30 * 1000
        onTriggered: netpulse.logOut()
        function reset() { if(running) restart() }
    }

    Connections {
        // reset the login timer when there's a screen touch
        target: application
        onScreenTouched: netpulseLoginTimer.reset()
    }

    AMixer {
        id: amixer
    }

    Text{
        id:versionNumber
        font.pointSize: 10 * main.screenScaleY
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.horizontalCenterOffset: demoModeText.visible ? -demoModeText.width/2 : 0
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 3 * main.screenScaleY
        color: "black"
        opacity: 0.5
        visible: !fullscreenVideoFrame.visible
        text:qsTr("Version: ") + application.appVersion + application.emptyString

        MouseArea
        {
            anchors.fill: parent
        }
    }

    Text {
        id: demoModeText
        visible: options.ignoreFaults || options.manufacturingTestModeEnabled
        font.pointSize: 10 * main.screenScaleY
        font.bold: true
        anchors.left: versionNumber.right
        anchors.leftMargin: 3 * main.screenScaleX
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 3 * main.screenScaleY
        color: options.manufacturingTestModeEnabled ? "#ef3d47" : "black"
        opacity: 0.5
        text: (options.manufacturingTestModeEnabled ? qsTr("Test Mode") : qsTr("Demo Mode")) + application.emptyString
    }

    StatisticManager{
        id:statisticManager

        onTotalDistanceChanged: errorManager.updateMileage(totalDistance)

        Component.onCompleted: {
            setNucleusObject(nucleus);
        }
    }

    MessagePopup {
        id: infoPopup
        dismissable: true
        dismissAnywhere: true
        isTransparent: true
    }

    MessageBanner {
        id: infoBanner

        z: 10000
        anchors.horizontalCenter: main.horizontalCenter
        y:0
        dismissable: true
        dismissAnywhere: true
        isTransparent: true
    }

    SimpleMessageBanner {
        id: viewContainerBanner

        z: 10000
        hideWhen: !main.isCSA && !main.isLateralTrainer && workoutScreen.container.currentViewName === "C360_VIEW"
        x:185
        y:170
        width: 1000 * main.screenScaleX
        dismissable: true
        dismissAnywhere: true
        isTransparent: true
    }

    Connections {
        target: machinecontrol

        onEstopChanged: {
            logger.logAlways("MachineControl: Safety Key " + (machinecontrol.estop ? "In" : "Out"))
        }

        onSafetyKeyQuickToggle: {
            miscStats.incrementUsage("Safety Key Toggle")
            logger.logAlways("MachineControl: Safety key quick toggle detected")
            logger.dumpLastLog()
        }
    }

    PopupManager{
        id: popupManager

        safetyKeyOut: !machinecontrol.estop && isTreadmill
        // hide pop-ups while in diagnostic screen and other situations
        hidePopup: main.state === "DiagnosticState" || main.state === "SetupWizardState"  || main.restartRequired || main.rebootRequired || options.ignoreFaults || main.firmwareUpdateRequired
        treadStopped: machinecontrol.treadmillSpeedActual < 1

        onEstopPopupTimedout: console.debug("estop timed out")

        Connections {
            target: machinecontrol

            onEstopChanged: {
                if(main.lowerSystem === "NUCLEUS" && estop === true && isTreadmill) {
                    popupManager.safetyKeyOut = false
                }
            }

            onSafetyKeyPulled: {
                if(main.lowerSystem === "NUCLEUS" && isTreadmill) {
                    popupManager.safetyKeyOut = true
                }
            }
        }
    }

    Connections {
        target: nucleusfirmwareloader
        onIsUpdatingChanged: errorManager.isNucleusFirmwareUpdating = nucleusfirmwareloader.isUpdating
    }

    ErrorManager{
        id:errorManager

        property bool fatalError: false
        property bool preventUse: false
        property bool degradedUse: false

        onErrorEvent: {
            if(options.ignoreFaults) {
                console.log("Ignoring error event: " + errorcode + " " + errormessage)
                return
            }

            if(isLogged && netpulseAvailable)
                dataExport.serviceEvent(errorcode,"");

            logger.logAlways("Error event: " + errorcode + " " + errormessage)
            logger.dumpLastLog()
            if(errorcode === "CN25")
                main.firmwareUpdateRequired = true

            if(severity == ErrorManager.ERROR_FATAL)
                fatalError = true
            if(severity >= ErrorManager.ERROR_PREVENTS_USE)
                preventUse = true
            else if(severity >= ErrorManager.ERROR_DEGRADES_USE)
                degradedUse = true

            if(!options.ignoreFaults) {
                var translatedErrorMessage = qsTranslate("ErrorManager",errormessage)
                popupManager.setErrorMessage(errorcode, translatedErrorMessage, severity)

                // briefly display new error at top of screen if pop-ups are hidden (as in diagnostic screens.)
                if(popupManager.hidePopup)
                    infoBanner.showMessage(popupManager.currentMessage, 5);

                if (errorcode === "X101" || errorcode === "X102") {
                    console.log("Stride Fault")
                    machinecontrol.strideFaulted = true
                }

            }
        }

        onNucleusNonRecoverableFaultTriggered: {
            preventUse = true;
            popupManager.setNucleusFaultTriggeredMessage(msg);
            if(main.state === "RunState") {
                workoutScreen.stopWorkout();
            }

            if(popupManager.hidePopup)
                infoBanner.showMessage(popupManager.currentMessage, 5);
        }

        onNucleusRecoverableFaultTriggered: {
            preventUse = true;
            popupManager.setNucleusFaultTriggeredMessage(msg);

            if(main.state === "RunState") {
                workoutScreen.stopButtonPressedHandler();
            }
        }

        onNucleusFaultRecovery: {
            preventUse = true;
            popupManager.setNucleusFaultRecoveringMessage(msg);
        }

        onNucleusFaultCleared: {
            preventUse = false;
            popupManager.reset()
        }

        onNucleusWarningTriggered: {
            infoBanner.showMessage(msg, 5)
        }

        onNucleusWarningCleared: {
            infoBanner.showMessage(msg, 5)
        }


        Component.onCompleted: {
            setNucleusObject(nucleus);
            setNucleusFaultInfoManager(nucleusFaultInfoMgr);
        }
    }

    WarningText {
        id: warningText
        enabled: (visible && opacity > 0.0)
        visible: main.isTft && main.isTreadmill && (!ipod_window.visible && !tv_window.visible)
        anchors { right: parent.right; rightMargin: 30 * main.screenScaleX; top: parent.top; topMargin: 105 * main.screenScaleY }
    }

    StorageMgr{
        id:storageMgr

    }

    RemoteAccessDialog {
        id: accessDialog
        property bool showMe: main.state === "HomeState" && remoteAccess.isEnabled
        onShowMeChanged: {
            if (showMe) {
                show()
            } else {
                hide()
            }
        }
        isTransparent: true
    }

    RemoteAccess {
        id: remoteAccess
    }

    SoftwareUpdatePrompt {
        id: softwareUpdatePrompt
        property bool deviceLoaded: storageMgr.usbDeviceLoaded
        prompt: qsTr("Update Software Now?") + application.emptyString

        x: 300 * main.screenScaleX
        y: 200 * main.screenScaleY

        onNoClicked: hide()
        Component.onCompleted: hide()

        onDeviceLoadedChanged: {
            if (main.state != "HomeState") {
                return
            }

            if (storageMgr.usbDeviceLoaded) {
                console.log("USB Device Loaded", storageMgr.usbDevicePath, storageMgr.usbBlockDevicePath)
                var hasPackages = storageMgr.checkForPackagesFile()
                console.log(hasPackages ? "Has Packages" : "No Packages")
                if (hasPackages) {
                    softwareUpdatePrompt.showPrompt()
                }
            } else {
                console.log("USB Device Removed", storageMgr.usbDevicePath, storageMgr.usbBlockDevicePath)
                if (softwareUpdatePrompt.visible) {
                    softwareUpdatePrompt.hide()
                }
            }
        }
    }

    Loader {
        id: screensaver
        z:1000
        anchors.fill: parent
        visible: false

        Text {
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.bottom: parent.bottom ; anchors.bottomMargin: 20 * main.screenScaleY
            font.pointSize: 40 * main.screenScaleY

            color: "white"
            z:99
            text: (main.isTft ? qsTr("Press Any Button to Begin") : qsTr("Touch to Begin")) + application.emptyString
        }

        function start() {
            visible=true
            screensaver.source = "Slideshow.qml"
            screensaver.item.zoomEffect = true
            screensaver.item.fadeEffect = true
            screensaver.item.slideInterval = 10000
        }
        function stop() {
            visible=false
            screensaver.source = ""
            if ((machineinfo.modelString === "----" && !setupWizard.isNucleus) || (setupWizard.isNucleus && !options.setupWizardCompleted)) {
                main.state = "SetupWizardState"
            } else {
                main.state = "HomeState"
            }
        }

        Connections {
            target: machinecontrol
            // wake up out of screen saver when pedaled.
            onPedalRPMChanged: {
                if(pedalRPM > 0) {
                    if(main.state === "ScreenSaverState" || main.state === "SleepModeState")
                        screensaver.stop()
                    else if (main.state === "HomeState") {
                        main.lcdBrightness = options.lcdBrightness
                        if (!options.screenSaveTimeout && options.dimDisplayTimeout)
                            dim_timer.restart()
                    }
                }

                if(workoutScreen.workoutActive && (machineinfo.productType != WorkoutProgram.TREADMILL)) {
                    // update the pace profile with current pace during a workout.
                    aWorkoutRunData.setRPM(pedalRPM)
                    workoutProgram.profile2.setProperty(workoutProgram.profile2.currentSegment, "segmentMagnitude", pedalRPM)
                }
            }

            onDisplayWattsChanged: { aWorkoutRunData.setWatts(displayWatts) ; workoutManager.setWatts(displayWatts) }
            onStrideLengthTargetChanged: {
                workoutManager.strideLevel = strideLengthTarget
            }
        }

        MouseArea {
            anchors.fill: parent
            onClicked: screensaver.stop()
        }
    }

    function quickStart(filename) {
        if (isTreadmill && machinecontrol.treadmillSpeedActual !== 0) {
            return
        }

        if (main.state === "SleepModeState")
            return

        if (softwareUpdatePrompt.active) return
        if (logUploadPrompt.visible) return
        if (blockUseMessage.visible) return
        if (accessDialog.visible) return
        if (main.quickStartLoading) // prevent multiple consecutive loads
            return
        main.quickStartLoading = true

        print("Quickstart " + (filename ? filename : ""))
        logoImage.cancelBlink()
        main.channelSelected = ""
        if( main.state === "HomeState" || main.state == "ScreenSaverState" || main.state === "MenuState" || main.state === "AllWorkoutsState") {
            if(main.state === "ScreenSaverState")
                screensaver.stop()
            tv_window.hide()
            ipod_window.close()

            if(isTreadmill) {
                if(!filename)
                    filename ="Tread_Quick_Start.xml";
            }
            else {
                if(!filename)
                    filename = "Quick_Start.xml"
            }
            workoutStats.incrementUsage(qsTr("Quick Start"))
            workoutProgram.loadWorkoutFileAsync("Program_Files/" + filename);
//            conditionalStart()
            aWorkoutRunData.programName = qsTr("QUICK START")
        }
    }

    function startWorkout(item){
        logoImage.cancelBlink()
        main.channelSelected = ""
        main.clearQLists();
        var programFile = item.program

        workoutStats.incrementUsage(item.label)

        if(item.program === "SAVED_WORKOUTS") {
            workoutProgram.reset()
            setupScreen.showSavedWorkouts()
            finishLoading(item)
        }
        else {
            state = "LoadState";
            loadingProgramNotice.programItem = item
            workoutProgram.loadWorkoutFileAsync("Program_Files/" + programFile)
        }

    }

    Connections {
        target: workoutProgram
        onParsingFileChanged: {
            // saved programs are handled by the saved programs browser, so skip them
            if (!workoutProgram.parsingFile && workoutProgram.customProgType === WorkoutProgram.UNDEFINED_PROGRAM_TYPE) {
                finishLoading(loadingProgramNotice.programItem)
            }
        }
    }

    function finishLoading(item) {
        if(!workoutProgram.isQuickStart){
            if (item.program !== "SAVED_WORKOUTS")
                setupScreen.setProgramName(item.label)
            aWorkoutRunData.programName = item.label

            main.state = "SetupState";
        }
        else {
            aWorkoutRunData.programName = qsTr("QUICK START")
            workoutStartManager.conditionalStart()
        }
    }

    function conditionalStart(){
        workoutManager.resetLevels();
        aWorkoutRunData.reset()
        workoutManager.setupWorkout()
        main.state = "RunState";
        netpulse.startWorkout()
        workoutScreen.start()
        if(machineinfo.productTypeString === "Treadmill"){
            testwarning.show()
        } else if (main.isCSA && workoutProgram.isC360Program) {
            // Allow intro video to play
        } else{
            workoutManager.start()
        }
    }

    TreadmillWorkoutCountdownWarning{
        id:testwarning
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter

        onJumpStart: {
            workoutManager.jumpStart()
        }

        onTimedOut: {
            workoutManager.start()
            hide()
        }
        onVisibleChanged: if(visible) {
                              delayTimer.start()
                          } else {
                              /* reset counter when hiding dialog */
                              countDown = 3
                          }
        onTriggered: buzzer.warningBeep()
        Component.onCompleted: hide()

        Connections {
            target: workoutScreen
            /* Hide the dialog, which stops count, when workout active status
             * changes to false during count down.
             */
            onWorkoutActiveChanged: {
                if(testwarning.visible) {
                    testwarning.hide()
                }
            }

           // onWorkoutStarted: {nucleus.workoutStart(); wahoo.startWorkout() }
           // onWorkoutStopped: {nucleus.workoutStop(); wahoo.stopWorkout(); testwarning.hide() }
           // onWorkoutPaused: {nucleus.workoutPause(); wahoo.pauseWorkout(); testwarning.hide()}
           //onWorkoutResumed: {nucleus.workoutResume(); wahoo.startWorkout()}

        }

        Timer{
            id: delayTimer
            interval: 350
            running: false
            repeat: false
            onTriggered: {
                testwarning.startTimeout(3)
                buzzer.warningBeep()
            }
        }
    }

    function clearQLists() {
        workoutManager.reset();
        setupScreen.clearListView(); // Use to reset setup items
        aWorkoutRunData.reset();
    }

    Connections {
        target: application
        onIsTftChanged: {
            selectWorkoutMenu()
            workout_menu.changeMenu(main.topmenu)
        }
    }

    Connections {
        target: machineinfo
        onModelStringChanged: {
            // Re-select workout menu
            selectWorkoutMenu()
            workout_menu.changeMenu(main.topmenu)

        }
    }

    function selectWorkoutMenu() {
        var Menus = isTft ? TftMenus : TouchMenus
        console.debug("Main: Select menu for " + machineinfo.productTypeString)
        switch(machineinfo.productTypeString) {
            case "Treadmill":
                main.topmenu = Menus.menu_tread_top()
                main.allWorkoutsMenu = main.isAlpineRunner ? Menus.menu_alpine_all() : Menus.menu_tread_all()
                break;
            case "Elliptical":
            //case "StrideAdjustElliptical":
                main.topmenu = Menus.menu_elliptical_top()
                main.allWorkoutsMenu = main.isCSA ? Menus.menu_csa_all() : Menus.menu_elliptical_all()
                break;
            case "Bike":
                main.topmenu = Menus.menu_bike_top()
                main.allWorkoutsMenu = Menus.menu_bike_all()
                break;
            case "Lateral Trainer":
                main.topmenu = Menus.menu_lateral_all()
                main.allWorkoutsMenu = Menus.menu_lateral_all()
                break
            default:
                main.topmenu = Menus.menu_elliptical_top()
                main.allWorkoutsMenu = Menus.menu_elliptical_all()
                break;
        }
        //console.log("topmenu", JSON.stringify(main.topmenu), main.isCSA)
        workout_menu.menu = main.topmenu
    }

    ImageButton {
        id: screen_lock_button
        visible: isTouch
        enabled: (visible && opacity > 0.0)
        anchors.top: parent.top
        anchors.topMargin: 14 * main.screenScaleY
        anchors.rightMargin: 30 * main.screenScaleX
        z:1
        upimage: main.graphicsPath + "LOCK_ICON_DISABLED.png"
        downimage: main.graphicsPath + "LOCK_ICON_DOWN.png"
        label: isDown ? qsTr("Hold...") + application.emptyString : ""
        label_y_offset: 10 * main.screenScaleY
        fontsize: 14 * main.screenScaleY
        label_color: "black"
        label_under: true
        onPressAndHold: screen_lock_overlay.open();
        anchors.right: parent.right;
        mouseAreaScale: 2.0
    }


    // screen lock which is opened when lock icon is pressed
    ScreenLock {
        id: screen_lock_overlay
        z:99
        lockTime: 20
        visible: false
    }

    Image {
        id: background
        z: -2
        visible: !fullscreenVideoFrame.visible
        anchors.fill: parent
        source: main.graphicsPath + "Background.png"

        Image {
            id: logoImage;
            enabled: (visible && opacity > 0.0)

            property bool blinking
            function startBlink() {
                logo_blink_timer.start()
                logoImage.blinking = true
            }
            function cancelBlink() {
                blinking = false
                logo_blink_timer.stop()
            }
            visible: main.isTouch || (!ipod_window.visible && !tv_window.visible)
            anchors.top: parent.top
            anchors.topMargin: 25 * main.screenScaleY
            anchors.left: parent.left
            anchors.leftMargin: 25 * main.screenScaleX

            source: main.graphicsPath + "TRU_Home_Logo.png"

            MouseArea {
//                 onClicked: main.state = "DiagnosticState"   // DEBUG: easy diag access
                onPressAndHold: logoImage.startBlink()
                anchors { top: parent.top; left: parent.left
                          topMargin: -25 * main.screenScaleY; leftMargin: -25 * main.screenScaleX }
                width: logoImage.width + 30 * main.screenScaleX
                height: logoImage.height + 30 * main.screenScaleY
                enabled: (main.state == "HomeState") && !(tv_window.opacity > 0) && !ipod_button.isDown
                Timer {
                    id: logo_blink_timer
                    property int blink_count: 0
                    running: false
                    repeat: true
                    interval: 1000
                    onTriggered: { logoImage.opacity = !logoImage.opacity ; blink_count++ }
                    onBlink_countChanged: {
                        if(blink_count > 10) {
                            logoImage.blinking = false
                            logoImage.opacity=1.0
                            blink_count=0;
                            logo_blink_timer.stop()
                        }
                    }
                } // end Timer
            } // end MouseArea for logo
        } // end Image True logo

        Image {
            id: alpineRunnerBadge
            visible: isAlpineRunner && logoImage.visible && (logoImage.opacity === 1)
            source: main.graphicsPath + "TI1000/ALPINE_homescreen_badge.png"
            anchors.verticalCenter: logoImage.verticalCenter
            anchors.left: logoImage.right
            anchors.leftMargin: 20 * main.screenScaleX
        }

    } // end Image background

    MouseArea {  // secret second mouse area to hold to get to diagnostics
        id: secretMouseArea
        z: logoImage.blinking ? 1000 : -10
        width: 100 * main.screenScaleX ; height: 100 * main.screenScaleY
        anchors { bottom: parent.bottom ; right: parent.right }
        onPressAndHold: {
            if( logoImage.blinking) {
                main.state = "DiagnosticState"
                logoImage.cancelBlink()
            }
        }
    }

    NavigationBar {
        id: navbar
        visible: isTouch
        enabled: (visible && opacity > 0.0)

        anchors.top: parent.top
        anchors.topMargin: 18 * main.screenScaleY
        anchors.right: parent.right
        anchors.rightMargin: 15 * main.screenScaleX

        onClicked: {
            if(state)
                main.state = state
            if(data)
                workout_menu.changeMenu( data, highlightIndex )
            workoutManager.reset();
            setupScreen.clearListView(); // Use to reset setup items
        }
    }

    Item {
        id: workoutStartManager
        property bool engaging: false

        function conditionalStart() {
            if(machineinfo.productTypeString === "Treadmill" && lowerSystem === "NUCLEUS") {
                if(engaging === false) {
                    engaging = true
                    machinecontrol.treadmillSafetyCircuitClose()
                } else {
                    main.conditionalStart()
                }
            } else {
                main.conditionalStart()
            }
        }

        Connections {
            target: machinecontrol

            onSafetyCircuitEngaged: {
                if(workoutStartManager.engaging === true) {
                    workoutStartManager.engaging = false
                    main.conditionalStart()
                }
            }

            onSafetyCircuitDisengaged: {
                if(workoutStartManager.engaging === true) {
                    workoutStartManager.engaging = false
                    main.quickStartLoading = false
                }
            }
            onSafetyKeyInserted: {
                if(workoutStartManager.engaging === false) {
                    workoutStartManager.engaging = true
                }
            }
        }

    }



    Loader {
        id: home_screen_loader
        sourceComponent: isTouch ? touch_home_screen_component : keyed_home_screen_component
        anchors.fill: parent
        anchors.rightMargin: 0
        anchors.bottomMargin: 0
        anchors.leftMargin: 0
        anchors.topMargin: 0
        z: 4
        opacity: 0
        enabled: (visible && opacity > 0.0)

        Component {
            id: touch_home_screen_component
            HomeScreen {
                id: touch_home_screen
                x: 0
                y: 0
                opacity: 1
                visible: !application.crashRecovery && !popupManager.estopShown && !errorManager.preventUse && !blockUseMessage.visible && !accessDialog.visible && isTouch
                /**********************************************************************

                  Truth table for buttonsEnabled
                  +-------------------+-------------------+------------++----------------+
                  | productTypeString | treadIsCalibrated | standalone || buttonsEnabled |
                  +-------------------+-------------------+------------++----------------+
                  |         ""        |     don't care    |   false    ||     false      |
                  |       "Bike"      |     don't care    |   false    ||     true       |
                  |    "Elliptical"   |     don't care    |   false    ||     true       |
                  |    "Treadmill"    |        true       |   false    ||     true       |
                  |    "Treadmill"    |        false      |   false    ||     false      |
                  |         ""        |     don't care    |    true    ||     false      |
                  |       "Bike"      |     don't care    |    true    ||     true       |
                  |    "Elliptical"   |     don't care    |    true    ||     true       |
                  |    "Treadmill"    |        true       |    true    ||     true       |
                  |    "Treadmill"    |        false      |    true    ||     true       |
                  +-------------------+-------------------+------------++----------------+

                 **********************************************************************/
                buttonsEnabled: !((machineinfo.productTypeString === "" && options.ignoreFaults == false) ||
                    (machineinfo.productTypeString === "Treadmill" && machinecontrol.isCalibrated == false && options.ignoreFaults == false) ||
                    (machineinfo.productTypeString === "" && options.ignoreFaults == true)) || options.manufacturingTestModeEnabled
                netpulseAvailable: main.netpulseAvailable
                showCardio360: machineinfo.productType === WorkoutProgram.ELLIPTICAL || machineinfo.productTypeString === "Lateral Trainer"
                showMonumentsButton: main.isAlpineRunner
                // hide the warnings and show mini-quickstart button if ipod or tv window open
                entertainmentShown: (tv_window.visible || ipod_window.visible) ? true : false
                onQuickstartPressed: {
                    quickStart()
                }
                onLoginPressed: main.state="LoginState"
                onRegisterPressed: main.state="RegisterState"
                onSignoutPressed: {
                    netpulse.logOut()
                    main.state="HomeState"
                }
            }
        }

        Component {
            id: keyed_home_screen_component
            KeyedHomeScreen {
                id: keyed_home_screen
                x: 0
                y: 0
                opacity: 1
                enabled: (visible && opacity > 0.0)
                visible: !application.crashRecovery && !popupManager.estopShown && !errorManager.preventUse && !blockUseMessage.visible && !accessDialog.visible && !isTouch
                hideBackground: ipod_window.visible || tv_window.visible
                /**********************************************************************

                  Truth table for buttonsEnabled
                  +-------------------+-------------------+------------++----------------+
                  | productTypeString | treadIsCalibrated | standalone || buttonsEnabled |
                  +-------------------+-------------------+------------++----------------+
                  |         ""        |     don't care    |   false    ||     false      |
                  |       "Bike"      |     don't care    |   false    ||     true       |
                  |    "Elliptical"   |     don't care    |   false    ||     true       |
                  |    "Treadmill"    |        true       |   false    ||     true       |
                  |    "Treadmill"    |        false      |   false    ||     false      |
                  |         ""        |     don't care    |    true    ||     false      |
                  |       "Bike"      |     don't care    |    true    ||     true       |
                  |    "Elliptical"   |     don't care    |    true    ||     true       |
                  |    "Treadmill"    |        true       |    true    ||     true       |
                  |    "Treadmill"    |        false      |    true    ||     true       |
                  +-------------------+-------------------+------------++----------------+

                 **********************************************************************/
                buttonsEnabled: !((machineinfo.productTypeString === "" && options.ignoreFaults == false) ||
                    (machineinfo.productTypeString === "Treadmill" && machinecontrol.isCalibrated == false && options.ignoreFaults == false) ||
                    (machineinfo.productTypeString === "" && options.ignoreFaults == true))
                netpulseAvailable: main.netpulseAvailable
                showCardio360: machineinfo.productType === WorkoutProgram.ELLIPTICAL || machineinfo.productTypeString === "Lateral Trainer"
                // hide the warnings and show mini-quickstart button if ipod or tv window open
                entertainmentShown: (tv_window.visible || ipod_window.visible) ? true : false
                onQuickstartPressed: {
                    quickStart()
                }
                onC360QuickstartPressed: {
                    quickStart("Cardio_360_Quick_Start.xml")
                }
                onC360QuickstartHeld: {
                    demoC360Notice.showWithTimeout(3);
                    main.quickStart("Cardio_360_Demo.xml")
                }
                onWorkoutFinderPressed: {
                    if (main.isCSA || main.isLateralTrainer || main.isAlpineRunner) {
                        main.state = "AllWorkoutsState"
                    } else {
                        main.state = "MenuState"
                    }
                }
                onLoginPressed: main.state="LoginState"
                onRegisterPressed: main.state="RegisterState"
                onSignoutPressed: {
                    netpulse.logOut()
                    main.state="HomeState"
                }

                MessagePopup {
                    id: demoC360Notice
                    message: qsTr("Cardio360 Demo") + application.emptyString
                }
            }
        }
    }

    // Display the workout program image while program loads from XML
    Item {
        id: loadingProgramNotice

        anchors.fill: parent
        property variant programItem : undefined
        opacity: 0
        enabled: (visible && opacity > 0.0)

        Image {
            id: loadingProgramImage
            anchors {  horizontalCenter: parent.horizontalCenter ; verticalCenter: parent.verticalCenter }
            source: {
                if (!loadingProgramNotice.programItem)
                    return ""
                if (options.modestyMode && loadingProgramNotice.programItem.modestyImage) {
                    return main.graphicsPath + loadingProgramNotice.programItem.modestyImage
                } else {
                    return main.graphicsPath + loadingProgramNotice.programItem.image
                }
            }
        }

        Text {
            id: loadingProgramLabel
            text: loadingProgramNotice.programItem ? qsTranslate("Menu", loadingProgramNotice.programItem.label) : ""
            font { pointSize: 14 * main.screenScaleY ; bold: true }
            anchors { horizontalCenter: loadingProgramImage.horizontalCenter
                top: loadingProgramImage.bottom
            }
        }

        Rectangle {
            id: progressBar
            color: "blue"
            height: 10 * main.screenScaleY
            width: (workoutProgram.parsingProgress / 100) * 360
            anchors { left: loadingProgramImage.left; leftMargin: 20 * main.screenScaleX; top: loadingProgramLabel.bottom; topMargin: 10 * main.screenScaleY }
            opacity: 0.5
        }
    }

    Loader {
        id: workout_menu_loader
        sourceComponent: isTouch ? touch_workout_menu_component : keyed_workout_menu_component
        anchors.fill: parent
        focus: main.state === "MenuState"
        opacity: 0
        enabled: (visible && opacity > 0.0)

        Component {
            id: touch_workout_menu_component

            WorkoutMenu {
                id: touch_workout_menu

                opacity: 1
                enabled: (visible && opacity > 0.0)
                visible: (tv_window.visible || ipod_window.visible) ? false : true

                onWorkoutSelected: {
                    navbar.push(main.graphicsPath + menu.navicon, "MenuState", menu)
                    startWorkout(item)
                }
                // signal emitted by workout menu when sub-menu selected.  menu parameter is a javascript dictionary
                onMenuSelected: {
                    // push item info onto the navigation stack. push(icon, state string, data=javascript dict)
                    // third parameter is returned in clicked(data) signal from navigation bar
                    navbar.push(main.graphicsPath + menu.navicon, "MenuState", menu)
                }

                ImageButton {  // All workouts button
                    id: all_workouts_button
                    z:5
                    visible: (workout_menu.menu && workout_menu.menu.label === main.topmenu.label) ? 1 : 0    // show all workouts button only on top menu
                    onClicked: { main.state = "AllWorkoutsState" }
                    anchors.bottom: parent.bottom
                    anchors.bottomMargin: 40 * main.screenScaleY
                    anchors.right: parent.right
                    anchors.rightMargin: 210 * main.screenScaleX
                    downimage: main.graphicsPath + "200/ShowAll_SM_Button_Down.png"
                    upimage: main.graphicsPath + "200/ShowAll_SM_Button_Disabled.png"
                }
                Text {
                    id: all_workouts_text
                    z: 5
                    width: 90 * main.screenScaleX
                    visible: all_workouts_button.visible
                    text: qsTr("SHOW ALL\nWORKOUTS") + application.emptyString
                    anchors {
                        left: all_workouts_button.right
                        leftMargin: -5 * main.screenScaleX
                        verticalCenter: all_workouts_button.verticalCenter
                        verticalCenterOffset: -5 * main.screenScaleY
                    }
                    font.pixelSize: 15 * main.screenScaleY
                    font.bold: true
                    wrapMode: Text.WordWrap
                }
            }
        }

        Component {
            id: keyed_workout_menu_component

            KeyedWorkoutMenu {
                id: keyed_workout_menu

                opacity: 1
                enabled: (visible && opacity > 0.0)
                visible: (tv_window.visible || ipod_window.visible) ? false : true

                onWorkoutSelected: {
                    navbar.push(main.graphicsPath + menu.navicon, "MenuState", menu, workout_menu.currentIndex)
                    startWorkout(item)
                }
                // signal emitted by workout menu when sub-menu selected.  menu parameter is a javascript dictionary
                onMenuSelected: {
                    // push item info onto the navigation stack. push(icon, state string, data=javascript dict)
                    // third parameter is returned in clicked(data) signal from navigation bar
                    navbar.push(main.graphicsPath + menu.navicon, "MenuState", menu, workout_menu.currentIndex)
                }

                //        ImageButton {  // All workouts button
                //            id: all_workouts_button
                //            z:5
                //            visible: false //(workout_menu.menu.label === main.topmenu.label) ? 1 : 0    // show all workouts button only on top menu
                //            onClicked: { main.state = "AllWorkoutsState" }
                //            anchors.bottom: parent.bottom
                //            anchors.bottomMargin: 40 * main.screenScaleY
                //            anchors.right: parent.right
                //            anchors.rightMargin: 190 * main.screenScaleX
                //            fontsize: 15 * main.screenScaleY
                //            downimage: main.graphicsPath + "200/ShowAll_SM_Button_Down.png"
                //            upimage: main.graphicsPath + "200/ShowAll_SM_Button_Disabled.png"
                //            label_x_offset: 90 * main.screenScaleX
                //            label: qsTr("SHOW ALL\nWORKOUTS") + application.emptyString
                //            label_under: false
                //            label_color: "black"
                //        }
            }

        }
    }

    WorkoutMenuAll {
        id: workouts_all

        onWorkoutSelected: {
            navbar.push(main.graphicsPath + menu.navicon, "AllWorkoutsState", undefined)
            startWorkout(item)
        }

        menu: main.allWorkoutsMenu
        anchors.fill: parent
        opacity: 0
        enabled: (visible && opacity > 0.0)
    }

    Item {  // side panel menu
        id: side_panel

        width: 100 * main.screenScaleX
        height: 300 * main.screenScaleY
        anchors { right: parent.right ; bottom: parent.bottom  }
        visible: main.isTouch && !popupManager.estopShown
        enabled: (visible && opacity > 0.0)

        Column { // column of icons
            id: column1
            anchors { bottom: parent.bottom; bottomMargin: 30 * main.screenScaleY; right: parent.right ; rightMargin: 10 * main.screenScaleX }
            width: parent.width
            spacing: 20 * main.screenScaleX
            height: childrenRect.height

           ImageButton { // TV Button
                id: tv_button ;
                opacity: !main.netpulseAvailable && tv.channelList.channels.count && !blockUseMessage.visible && !accessDialog.visible ? 1.0 : 0.0
                enabled: opacity > 0
                toggle: true
                onClicked: ipod_button.setDown(false)
                upimage: main.netpulseAvailable ? main.graphicsPath + "TV_ICON_ONDEMAND_DISABLED.png" : main.graphicsPath + "TV_ICON_DISABLED.png"
                downimage: main.netpulseAvailable ? main.graphicsPath + "TV_ICON_ONDEMAND_DOWN.png" : main.graphicsPath + "TV_ICON_DOWN.png"
                anchors.horizontalCenter: parent.horizontalCenter;
                mouseAreaScale: 1.25
            }

            ImageButton { // iPod button
                id: ipod_button ;
                opacity: !main.netpulseAvailable && !blockUseMessage.visible && !accessDialog.visible ? 1.0 : 0.0
                enabled: opacity > 0
                toggle: true
                onClicked: tv_button.setDown(false)
                upimage: main.graphicsPath + "ITUNES_ICON_DISABLED.png"
                downimage: main.graphicsPath + "ITUNES_ICON_DOWN.png"

                anchors.horizontalCenter: parent.horizontalCenter
                mouseAreaScale: 1.25
            }

            ImageButton { // language menu icon
                id: language_button;
                onVisibleChanged: {
                    if (visible && _hideLanguageButton)
                        visible = false
                }
                upimage: main.graphicsPath + "LANGUAGE_ICON_DISABLED.png"
                downimage: main.graphicsPath + "LANGUAGE_ICON_DOWN.png"
                anchors.horizontalCenter: parent.horizontalCenter
                mouseAreaScale: 1.25
                onClicked: {
                    main.state = "LanguageMenuState"
                }
            }
        }
    } // end side_panel Item

    ChannelMenu {
        id: tv_menu
        property bool menuOpen: false
        width: 1000 * main.screenScaleX
        height: (workoutScreen.workoutActive ? 406 : 510) * main.screenScaleY
        anchors.left: parent.left
        anchors.top: parent.top
        anchors.leftMargin: 183 * main.screenScaleX
        anchors.topMargin: 163 * main.screenScaleY
        z: 97
        opacity: menuOpen ? 1 : 0
        enabled: (visible && opacity > 0.0)

        channels: tv.channelList.channels
        currentChannel: tv.currentChannel
        visible: true
        onChannelSelected: {
            menuOpen = false

            if(id) {
                main.channelSelected = id
                tv.tuneChannelID(id)
            }
        }
        onClose: {
            menuOpen = false
            if(tv.currentChannel < 0)
                tv_window.hide()
        }

        Behavior on opacity {
            NumberAnimation { duration: 200; easing.type: Easing.InOutQuad }
        }
    }

    Connections {
        target: main.isTft && (main.state === "MenuState" || main.state === "SetupState" || main.state === "AllWorkoutsState" || "WorkoutSummaryState") ? keymapper : null

        onPressAndHold: {
            main.debugKeys("press and hold", "KeyedWorkoutMenu", functionIndex)
            if (functionIndex === 18 || functionIndex === 6) // 18=STOP key; 6=Center pod STOP key
                main.state = "HomeState"
        }
    }

    Connections {
        target: main.state === "HomeState" && !ipod_window.visible && !tv_window.visible ? keymapper : null

        onUpButtonPressed: {
            main.debugKeys("upButtonPressed", "main")
            volume_controls.volumeUp();
        }

        onDownButtonPressed: {
            main.debugKeys("downButtonPressed", "main")
            volume_controls.volumeDown();
        }

        onPressAndHold: {
            main.debugKeys("press and hold", "main", functionIndex)
            if (functionIndex === 21) { // double left arrow <<
                buzzer.secretBeep(0)
                logoImage.startBlink()
            } else if (functionIndex === 16 || functionIndex === 24) { // ENTER or MINUS
                if (logoImage.blinking) {
                    buzzer.secretBeep(1)
                    main.state = "DiagnosticState"
                    logoImage.cancelBlink()
                }
            }
        }
    }

    AudioManager {
        id: audioManager
        debug: true
        tvPlaying: tv.playing && !tv.isIpodInput
        ipodPlaying: false
        ipodPlayingVideo: false
        netpulsePlaying: netpulse.playing
        browserPlaying: browserAudioWatcher.soundIsPlaying
        btconnected: bluetoothAdapter.connected
        onAudioSourcesChanged: print("audio sources: " + audioSources)
    }

    Process {
        id: browserAudioWatcher
        property bool soundIsPlaying: false

        program: "scripts/watch_browser.sh"

        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim()
            console.log("browserAudioWatcher stdout", data)
            if (data === "yes") {
                browserAudioWatcher.soundIsPlaying = true
            } else {
                browserAudioWatcher.soundIsPlaying = false
            }
        }

        onFinished: {
            console.log("browserAudioWatcher finished")
            browserAudioWatcher.soundIsPlaying = false
        }

        onSoundIsPlayingChanged: {
            if (soundIsPlaying)
                audioManager.browserRequest()
        }
    }

    Process {
        id: browserMute
        program: "scripts/browsermute.sh"

        function setMute(flag) {
            browserMute.arguments = [flag]
            browserMute.start()
        }
        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim()
            console.log("output from browsermute:",data)
        }
    }

    Process {
        id: vlcMute
        program: "scripts/vlcmute.sh"

        function setMute(flag) {
            vlcMute.arguments = [flag]
            vlcMute.start()
        }
        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim()
            console.log("output from vlcMute:",data)
        }
    }

    Connections {
        target: audioManager
        onBrowserAllowedChanged: {
            browserMute.setMute(! audioManager.browserAllowed)
        }

        onVideoSourceChanged: {
            console.log("Video Source is: " + audioManager.videoSource)
        }

        onTvVideoAllowedChanged: {
            muteTimer.stop()
            muteTimer.start()
        }
    }

    Timer{
        id: muteTimer
        running: false
        interval: 1000
        onTriggered: {
            running = false
            if(audioManager.tvVideoAllowed || audioManager.ipodVideoAllowed || (audioManager.ipodAllowed && bluetoothAdapter.connectedHeadset)){
                console.log("Unmuting VLC")
                vlcMute.setMute(0)
            } else {
                console.log("Muting VLC")
                vlcMute.setMute(1)
            }
        }
    }

    Connections {
        target: audioManager

        onBtAllowedChanged: {
            if (audioManager.btAllowed) {
                btMute.setMute(0)
            } else {
                btMute.setMute(1)
            }
        }
    }

    Process {
        id: btMute
        program: "scripts/btmute.sh"

        function setMute(flag) {
            btMute.arguments = [flag]
            btMute.start()
        }
        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim()
            console.log("output from btmute:",data)
        }
    }

    BluetoothDeviceSelectionDialog{        
        id: btDeviceSelectionDialog
        isTop: true
        height: main.height * 0.42
        width: main.width * 0.60
        anchors.horizontalCenter: main.horizontalCenter
        anchors.verticalCenter: main.verticalCenter
        pairingAllowed: !(bluetoothAdapter.pairing || bluetoothAdapter.connecting)

        onClosed: btDeviceSelectionDialog.hide()
        onHeadsetConnected: btDeviceSelectionDialog.hide()
        onHeadsetDisconnected: btDeviceSelectionDialog.hide()

    }

    HeartRateDeviceSelectionDialog{
        id: heartRateDeviceSelectionDialog
        isTop: true
        height: main.height * 0.42
        width: main.width * 0.60
        anchors.horizontalCenter: main.horizontalCenter
        anchors.verticalCenter: main.verticalCenter
        isLowEnergy: true

        onClosed: hide()
        onHeadsetConnected: hide()
        onHeadsetDisconnected: hide()

    }



    VolumeControls {
        id: volume_controls

        property bool canShow: main.isTouch && (main.state == "HomeState" || main.state === "RunState" || ipod_window.visible)
        opacity: ( canShow && audioManager.audioPlaying ) ? 1.0 : 0.0
        enabled: (visible && opacity > 0.0)
        visible: !fullscreenVideoFrame.visible
        anchors.verticalCenter: parent.verticalCenter
        anchors.verticalCenterOffset: 30 * main.screenScaleY
        anchors.left: parent.left
        anchors.leftMargin: 32 * main.screenScaleX
        showHeadsetButton: !options.netpulseEnabled

        // synchronize the control display with actual volume if changed elsewhere
        Connections {
            target: audioManager
            onVolumeChanged: {
                if(audioManager.volume != volume_controls.percent)
                    volume_controls.vol_setting = audioManager.volume * (volume_controls.vol_max - volume_controls.vol_min)/100
            }

            onMuteChanged: volume_controls.muted = audioManager.mute

        }

        onBluetoothHeadsetButtonClicked: {
            if(bluetoothAdapter.connectedHeadset === true){
                bluetoothAdapter.disconnectHeadset()
            } else {
                btDeviceSelectionDialog.show()
                btDeviceSelectionDialog.state = "list"
            }
        }

        onMutedChanged: audioManager.setMute(muted)

        onPercentChanged: audioManager.setVolume(percent)

        onSourceButtonClicked: {
            print("Audio Source Button Pressed.")
            audioManager.nextSource()
        }
        showSourceButton: audioManager.audioSources.length > 1 && !audioManager.netpulsePlaying && !bluetoothAdapter.connectedHeadset

        sourceIcon: {
            if (audioManager.ipodAllowed || audioManager.btAllowed || audioManager.ipodVideoAllowed)
                return "netpulse/audio_button_iPod.png"
            else if (audioManager.tvAllowed)
                return "netpulse/audio_button_TV.png"
            else if (audioManager.virtActiveAllowed)
                return "netpulse/audio_button_VA.png"
            else if (audioManager.browserAllowed)
                return "netpulse/audio_button_WEB.png"
            else
                return ""
        }

        Behavior on opacity { NumberAnimation { duration: 200; easing.type: Easing.InOutQuad } }
    }

    Television {
        id: tv

        onClicked: tv.fullscreen = tv.fullscreen ? false: true
        muted: false
        verbose: false

        onUsbResetFailedChanged: {
//            if(usbResetFailed) {
//                main.rebootRequired = true
//            }
        }

        Connections {
            target: audioManager
            onTvVideoAllowedChanged: {
                if(audioManager.tvVideoAllowed) {
                    tv.tuneChannelID(tv.currentChannel)
                    tv.on = true
                }
                else {
                    tv.stop()
                }
            }

            onTvAllowedChanged: {
                tv.muted = !audioManager.tvAllowed
                print("tv muted:" + audioManager.tvAllowed)
            }

        }

    }

    ModalDialog {
        id: rebootMessage

        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter

        blockKeys: true
//        property bool showPopup: main.state === "HomeState" && main.rebootRequired && tv.usbResetFailed
        property bool showPopup: false
        onShowPopupChanged: {
            if (showPopup){
                show()
                rebootMessage.startTimeout(5)
            } else
                hide()
        }

        onTimedOut: {
            main.startReboot()
            hide()
        }

        onVisibleChanged: {
            if(!visible) {
                // reset counter when hiding dialog
                rebootMessage.countDown = 5
            }
        }
        onTriggered: buzzer.warningBeep()
        Component.onCompleted: hide()
//        onCountDownChanged: if(countDown == 0) main.startReboot()

        BorderImage {   // pop-up dialog background
            id: border2

            anchors.horizontalCenter: rebootMessage.horizontalCenter
            anchors.verticalCenter: rebootMessage.verticalCenter
            width: 800 * main.screenScaleX
            height: 350 * main.screenScaleY
            source: main.graphicsPath + "Pop_Up_Lrg_Grey.png"
            border { top: 152 * main.screenScaleY; bottom: 165 * main.screenScaleY; left: 145 * main.screenScaleX; right: 144 * main.screenScaleX }

            Text {
                id: warningText2
                x: 149 * main.screenScaleX
                y: 35 * main.screenScaleY
                width: 501 * main.screenScaleX
                height: 99 * main.screenScaleY
                color: "#ffffff"
                text: qsTr("TV Error Detected.") + "\n" + qsTr(" Console Rebooting in ...") + application.emptyString
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.top: parent.top
                anchors.topMargin: 35 * main.screenScaleY
                horizontalAlignment: Text.AlignHCenter
                wrapMode: Text.WordWrap
                font.bold: true
                font.pixelSize: 40 * main.screenScaleY
            }

            Text { // show count down to timeout of prompt (started by startTimeout(seconds) method of ModalDialog superclass
                id: countdownText2

                x: 350 * main.screenScaleX
                y: 168 * main.screenScaleY
                color: "#ffffff"

                // display count down (with a leading 0 if remaining seconds < 10
                text: ":" + ((rebootMessage.countDown.toString().length < 2) ? "0" : "") + rebootMessage.countDown
                font.bold: true
                font.pixelSize: 96 * main.screenScaleY

                anchors.horizontalCenter: parent.horizontalCenter
                anchors.bottom: parent.bottom
                anchors.bottomMargin: 73 * main.screenScaleY
            }
        }
    }

    function startReboot () {
        if(tv.usbResetFailed) {
//            tvTestHud.usbhubcount++
            console.log("TV Error Stats:  Channel Change Count: " + tvTestHud.count +  ", USB Errors Detected: " + tv.numUSBErrors + ", VLC Error state: " + tv.vlcErrorOccurred)

        }
        console.log("************* Calling reboot!")
        rebootConsole.start()
    }

    Process { id: rebootConsole
        program: "scripts/export-diagnostics/reboot.sh"

        onStarted: {
            console.log("Reboot started")
        }

        onReadyReadStandardOutput: {
            var output = String(readAllStandardOutput())
            output = output.trim()
            console.log( "reboot process (STDOUT):" + output)
        }

        onReadyReadStandardError: {
            var output = String(readAllStandardError())
            output = output.trim()
            console.log( "reboot process (STDERR):" + output)
        }
    }

    property alias sapManager: sapLoader.item
    Loader {
        id: sapLoader
        sourceComponent: sapManagerComponent

        function reload() {
            sapLoader.sourceComponent = undefined
            sapLoader.sourceComponent = sapManagerComponent
        }


        Component {
            id: sapManagerComponent
            Item {
                property alias item: manager
                SapManager {
                    id: manager
                    //        onChannelAdded: {
                    //            console.log("channel added:", mrl, name)
                    //            var item = {}
                    //            item.id = mrl
                    //            item.mrl = mrl
                    //            item.name = name
                    //            //item.number = 0
                    //            tv.channelList.addChannel(item)
                    //        }

                    //        onChannelRemoved: {
                    //            console.log("channel removed:", mrl, name)
                    //            var id = mrl
                    //            tv.channelList.removeChannel(id)
                    //        }

                    onChannelFound: {
                    }
                }
            }

        }
    }


    Timer {
        running: true; interval: 3000; repeat: true
        onTriggered: {
//            sapManager.printSources()
        }
    }

    TvRecordingManager {
        id: tvRecordingManager
    }

    TvRecordingMessage {
        id: tvRecordingMessage
        visible: tvRecordingManager.recording || tvRecordingManager.copying
        z: 9999999
    }


    TelevisionWindow {  // pop-up TV window available from main screen only
        id: tv_window

        LayoutMirroring.enabled: main.rightToLeft
        LayoutMirroring.childrenInherit: true

        function hide() {
            if (main.isTouch)
                tv_button.isDown = false
            else if (main.isTft)
                home_screen.setTvButton(false)
        }
        onClose: tv_window.hide()

        onVisibleChanged: {
            if (visible) {
                if (options.cabEnabled) {
                    var firstChannel = tv.channelList.getChannelItem(0)
                    if (firstChannel !== null) {
                        console.log("CAB: Autotuning to", firstChannel.id)
                        tv_menu.channelSelected(firstChannel.id)
                    }
                } else {
                    tv_menu.menuOpen = true
                }
                if (tv.currentChannel >= 0) {
                    audioManager.tvRequest()
                }

                print("tv window turning on tv")
                tv.on = true
            }
            else {
                if(!ipod_window.visible) {
                    print("tv window turning off tv")
                    tv.on = false
                }
                tv_menu.menuOpen = false
            }
        }

        Connections {
            target: tv_window.visible ? audioManager : null
            //onTvAllowedChanged: tv.muted = !audioManager.tvAllowed
        }

        Connections {
            target: main.isTft && home_screen.tvSelected && !tv_menu.menuOpen ? keymapper : null
            ignoreUnknownSignals: true

            onUpButtonPressed: {
                main.debugKeys("upButtonPressed", "TelevisionWindow", "(in main.qml)")
                volume_controls.volumeUp();
            }

            onDownButtonPressed: {
                main.debugKeys("downButtonPressed", "TelevisionWindow", "(in main.qml)")
                volume_controls.volumeDown();
            }
        }

        // expire current channel selected from home screen after 5 minutes with tv off
        Timer {
            running: (main.state !== "RunState") && (tv.currentChannel >=0)
            interval: 5 * 60 * 1000
            onTriggered: {
                tv.currentChannel = -1
                tv_window.hide()
            }
        }

        anchors { horizontalCenter: main.horizontalCenter; top: main.top; topMargin: 140 * main.screenScaleY }
        opacity: tv_button.isDown || (isTft && home_screen.tvSelected) ? 1.0 : 0.0
        enabled: (visible && opacity > 0.0)
        visible: opacity > 0


        z:4  // place the TV window on top of main screen controls

        Behavior on opacity { NumberAnimation { duration: 200; easing.type: Easing.InOutQuad } }
    }

    // Full-screen Video Frame, for TV or video workouts, with data overlay
    Rectangle {
        id: fullscreenVideoFrame

        property QtObject videoItem  // the currently hosted fullscreen player

        // The item passed here must have a "fullscreen" property
        function showFullscreen(item) {
            item.fullscreen = true
            fullscreenSurface.player = item
            fullscreenSurface.aspect = item.aspect
            fullscreenVideoFrame.visible = true
            fullscreenVideoFrame.videoItem = item
        }

        function endFullscreen() {
            if(fullscreenVideoFrame.videoItem) {
                fullscreenVideoFrame.videoItem.fullscreen = false
                fullscreenVideoFrame.videoItem = null
                fullscreenSurface.player = null
            }
            fullscreenVideoFrame.visible = false
        }

        anchors.fill: parent
        visible: fullscreenVideoFrame.videoItem != null
        color: "black"
        z: tv_menu.z - 1

        VLCSurface {
            id: fullscreenSurface

            playing: fullscreenVideoFrame.visible
            anchors.fill: parent
            DiagnosticOverlay {
                id: diagOverlay
            }
        }

        MouseArea {
            // This mouse area is masked by the VideoDataOverlay when shown,
            // so click screen to minimize is only available in a non-workout fullscreen TV mode
            onClicked: {
                fullscreenVideoFrame.endFullscreen()
            }
            anchors.fill: parent
        }

        VideoDataOverlay {
            id: videoOverlay

            visible: (main.state === "RunState")
            isTreadmill: main.isTreadmill
            leftSideDisplay: workoutScreen.sideDataDisplay.leftDisplay
            rightSideDisplay: workoutScreen.sideDataDisplay.rightDisplay

            showHR: heartRateManager.heartRateAvailable
            heartRate: heartRateManager.heartRate
            incline: workoutManager.treadInclineGUI
            speed: workoutScreen.unitState == "ENGLISH" ? workoutManager.treadSpeedGUI : workoutManager.treadSpeedMetricGUI.toFixed(1)
            speedUnits: (workoutScreen.unitState === "ENGLISH" ? qsTr("MPH") : qsTr("KPH") ) + application.emptyString
            rpm: machinecontrol.pedalRPM
            level: workoutManager.workloadLevel
            // show the channel change button only if it is the TV that is fullscreen
            showChannelButton: tv.fullscreen && !tv.isIpodInput
            opacity: 0.5
            enabled: (visible && opacity > 0.0)
            anchors.fill: parent
            showData: (main.state === "RunState")
            onScreenButtonPressed: fullscreenVideoFrame.endFullscreen()
            onChannelButtonPressed: {
                if (netpulseAvailable) {
                    if(netpulse.visible)
                        netpulse.hide()
                    else
                        netpulse.showPage("tv")
                }
                else
                    tv_menu.menuOpen = (tv_menu.menuOpen ? false : true)
            }
            onStopButtonPressed: {
                console.log("VideoOverlay: Stop button pressed")
                workoutScreen.stopButtonPressedHandler()
            }
        }
    }

    LanguageMenu {
        id: language_menu
        opacity: 0
        enabled: (visible && opacity > 0.0)
        z:99
        anchors.fill: parent
        focus: main.state === "LanguageMenuState"

        onLanguageSelected: {
            navbar.clear()
            if(lang) {
                application.selectLanguage(lang)
                if(netpulseAvailable)
                    netpulse.setLanguage(lang)
            }
            main.state = "HomeState"
        }
    }

    // return to default language after a period of inactivity in the home screen
    Timer {
        id: lang_timer
        running: main.state === "HomeState"
        interval: 5 * 60 * 1000
        onTriggered: {
            if (main.state === "HomeState") {
                application.selectLanguage(options.defaultLanguage) // reset language after workout
                if(netpulseAvailable)
                    netpulse.setLanguage(options.defaultLanguage)
            }
        }
    }

    Item {
        id: tvTestHud

        property alias channelChangeTime: channelChangeTimeSlider.value
        property int usbhubcount: 0
        property int count: 0
        property ListModel tuneTimeModel: tv.tuneTimes

        opacity: 0.5
        visible: enabled
        enabled: options.autoChannelChangeInterval > 99
        width: childrenRect.width
        height: childrenRect.height
        anchors { right: parent.right; rightMargin: 25; verticalCenter: parent.verticalCenter }
        z:99
        Rectangle {
            id: tvTestItemRect
            color:"green"
            width: childrenRect.width
            height: childrenRect.height

            Column {
                spacing: 3
                Text { text: "Time: " + (tv.tuneTimes.count > 0 ? tv.tuneTimes.get(tv.tuneTimes.count-1).time/1000.0  + "s" : "") }
                Row {
                    id: barGraph
                    height: 50
                    width: 140
                    spacing: 1
                    Repeater {
                        model: tvTestHud.tuneTimeModel
                        delegate: Component {
                            Rectangle {
                                height: barGraph.height * (model.time / 5000.0)
                                width: barGraph.width / tvTestHud.tuneTimeModel.numberTimesToStore
                                anchors { bottom: parent ? parent.bottom : undefined }
                            }
                        }
                    }
                }

                Text { visible: true; text: "Interval: " + (channelChangeTimeSlider.off ? "OFF" : tvTestHud.channelChangeTime) }
                Text { visible: true; text: "Count: " + tvTestHud.count }

                SliderControl{
                    id: channelChangeTimeSlider
                    property bool off: channelChangeTimeSlider.value === 0 //channelChangeTimeSlider.maxValue
                    thumbImage: "graphics/205/IPOD_Red_Dot.png"
                    value: options.autoChannelChangeInterval //maxValue
                    width: 150 * main.screenScaleX
                    height: 30 * main.screenScaleY
                    minValue: 0
                    maxValue: 600 * 1000
                    stepValue: 500
                    showValue: false
                    onValueChanged: {options.autoChannelChangeInterval = value}
                }

                Text { text: "USB Faults Detected: " + tv.numUSBErrors }
                Text { text: "USB Hub Resets Needed: " + tvTestHud.usbhubcount }
                Text { text: "USB Resetting: " + tv.usbReseting }
                Text { text: "USB Reset Failed: " + tv.usbResetFailed }
                Text { text: "VLC Error Occured: " + tv.vlcErrorOccurred }
            }
        }
    }

    Item {
        id: debugOverlay

        property string asources: ((audioManager.ipodPlaying ? "ip " : "") +
                                  (audioManager.ipodPlayingVideo ? "iv " : "") +
                                  (audioManager.virtActivePlaying ? "va " : "") +
                                  (audioManager.tvPlaying ? "tv " : "") +
                                  (audioManager.browserPlaying ? "web " : "") +
                                  (audioManager.netpulsePlaying ? "np " : "") +
                                  (audioManager.btconnected ? "bt " : ""))

        property string aallowed: ((audioManager.ipodAllowed ? "ip " : "") +
                                  (audioManager.ipodVideoAllowed ? "iv " : "") +
                                  (audioManager.btAllowed ? "bt ": "") +
                                  (audioManager.virtActiveAllowed ? "va " : "") +
                                  (audioManager.virtActiveVideoAllowed ? "vv " : "") +
                                  (audioManager.tvAllowed ? "tv " : "") +
                                  (audioManager.tvVideoAllowed ? "tvv " : "") +
                                  (audioManager.browserAllowed ? "web " : "") +
                                  (audioManager.netpulseAllowed ? "np " : "") +
                                  (audioManager.netpulseVideoAllowed ? "npv " : ""))

        opacity: 0.5
        visible: false
        x: 20 * main.screenScaleX
        y: 480 * main.screenScaleY
        z:99
        Rectangle {
            color:"gray"
            width: childrenRect.width
            height: childrenRect.height

            Column {
                spacing: 3 * main.screenScaleX
                Text { visible: true; text: "audio:" + debugOverlay.asources }
                Text { visible: true; text: "allowed:"+ debugOverlay.aallowed}
                Text { visible: true; text: "mux:"+ audioManager.currentMuxSource}
                Text { visible: isTreadmill ; text: "target incline:" + machinecontrol.treadmillInclineTarget / 10.0}
                Text { visible: isTreadmill ; text: "actual incline:" + machinecontrol.treadmillInclinePercentGradeActual / 10.0}
                Text { visible: isTreadmill ; text: "target speed:" + machinecontrol.treadmillSpeedTarget / 10.0}
                Text { visible: isTreadmill ; text: "actual speed:" + machinecontrol.treadmillSpeedActual / 10.0}
                Text { visible: !isTreadmill ; text: "Workload:" + machinecontrol.workload }
                Text { visible: true; text: "time warp:" + timeFactorControl.value + "x"}

                SliderControl{
                    id: timeFactorControl
                    thumbImage: main.graphicsPath + "205/IPOD_Red_Dot.png"
                    value: main.timeFactor
                    width: 150 * main.screenScaleX
                    minValue: 0
                    maxValue: 30
                    showValue: false
                    onValueChanged: main.timeFactor = value
                }
            }
        }
    }

    Rectangle {
        id: languageDebug
        visible: false
        x: 20 * main.screenScaleX
        y: 400 * main.screenScaleY
        color: "#80659d32"
        width: 200 * main.screenScaleX
        height: childrenRect.height
        z:99999999999999999999
        Column {
            spacing: 2 * main.screenScaleX
            Repeater {
                model: [
                    { lang: "en", caption: qsTranslate("LanguageMenu", "ENGLISH") + application.emptyString } ,
                    { lang: "ru", caption: qsTranslate("LanguageMenu", "RUSSIAN") + application.emptyString } ,
                    { lang: "fr", caption: qsTranslate("LanguageMenu", "FRENCH") + application.emptyString } ,
                    { lang: "zh", caption: qsTranslate("LanguageMenu", "CHINESE") + application.emptyString } ,
                    { lang: "de", caption: qsTranslate("LanguageMenu", "GERMAN") + application.emptyString } ,
                    { lang: "nl", caption: qsTranslate("LanguageMenu", "DUTCH") + application.emptyString } ,
                    { lang: "ar", caption: qsTranslate("LanguageMenu", "ARABIC") + application.emptyString } ,
                    { lang: "es", caption: qsTranslate("LanguageMenu", "SPANISH") + application.emptyString } ,
                    { lang: "pt", caption: qsTranslate("LanguageMenu", "PORTUGUESE") + application.emptyString } ,
                    { lang: "ko", caption: qsTranslate("LanguageMenu", "KOREAN") + application.emptyString } ,
                    { lang: "ja", caption: qsTranslate("LanguageMenu", "JAPANESE") + application.emptyString } ,
                    { lang: "it", caption: qsTranslate("LanguageMenu", "ITALIAN") + application.emptyString } ,

                ]
                delegate: Component {
                    Text {
                        font { pixelSize: 16 * main.screenScaleY }
                        font.bold: application.currentLanguage === modelData.lang
                        color: "white"
                        text: modelData.caption
                        MouseArea {
                            anchors.fill: parent
                            onClicked: {
                                var lang = modelData.lang
                                application.selectLanguage(lang)
                                if(netpulseAvailable)
                                    netpulse.setLanguage(lang)
                            }
                        }
                    }
                }
            }
        }

    }

    // Keys to control the simulator
    Keys.onPressed: {
        switch(event.key){
            case Qt.Key_Q: { // fake stride-up for testing
                keymapper.keyPressHandler(8)
                break
            }
            case Qt.Key_A: { // fake stride-down for testing
                keymapper.keyPressHandler(3)
                break
            }
            case Qt.Key_R: { // Ctrl+r allows recording a TV stream
                if (event.modifiers & Qt.ControlModifier) {
                    console.log("ctrl-r pressed")
                    if (!tvRecordingManager.recording) {
                        tvRecordingManager.startRecording()
                    }
                }
                break
            }
            case Qt.Key_S: { // toggle on/off simulation of random pedaling / heart rate
                pedalSim.simulating = !pedalSim.simulating
                break;
            }
            case Qt.Key_G: {  // toggle holding heartrate grips
                pedalSim.holdingGrips = !pedalSim.holdingGrips
                break;
            }
            // must have simulation turned off to use the following controls
            case Qt.Key_BracketRight: { // ']' increase heartrate
                pedalSim.heartRate++
                break;
            }
            case Qt.Key_BracketLeft: { // '[' decrease heartrate
                pedalSim.heartRate--
                break;
            }
            case Qt.Key_Period: { // '.' increase pedal speed
                pedalSim.pedalRPM++
                break;
            }
            case Qt.Key_Comma: { // ',' decrease pedal speed
                pedalSim.pedalRPM--
                break;
            }
            case Qt.Key_AsciiTilde: // '~' toggle debug HUD
                debugOverlay.visible = !debugOverlay.visible
                break;
            case Qt.Key_QuoteDbl: {
                languageDebug.visible = !languageDebug.visible
                break
            }
            case Qt.Key_C: // 'C' toggle show mouse cursor
                application.showCursor = !application.showCursor
                break;
            case Qt.Key_O: // 0 key also takes screen shot
            case Qt.Key_P: {// 'P' take screen shot (photo)
                buzzer.buttonPress()
                print("taking screenshot " + main.screenShotCount)
                application.screenShot("/media/usb/screenshot_" + main.screenShotCount + ".png")
                infoBanner.showMessage("Screenshot " + main.screenShotCount + " saved.", 3);
                main.screenShotCount++
                break;
            }
            case Qt.Key_K: { // toggle safety key simulation
                pedalSim.safetyKeyIn = !pedalSim.safetyKeyIn
                print("estop:" + pedalSim.safetyKeyIn)
                break;
            }
            case Qt.Key_D: { // quick diagnostic entry
                main.state = "DiagnosticState"
                break;
            }
            case Qt.Key_T: {
                if (event.modifiers & Qt.ShiftModifier) {
                  tvTestHud.visible = !tvTestHud.visible
                } else {
                  application.isTft = !application.isTft
                }
                break
            }
        }
    }

    Keys.onReleased: {
        switch (event.key) {
            case Qt.Key_Q: { // fake stride-up for testing
                keymapper.keyReleaseHandler(62)
                break
            }
            case Qt.Key_A: { // fake stride-down for testing
                keymapper.keyReleaseHandler(63)
                break

            }
        }
    }

    Connections {
        target: options
        onSimulationModeChanged: {
            console.debug("Simulation mode changed: " + options.simulationMode)
        }
    }

    Connections {
        target: heartrate
        onTelemetryHeartRateChanged: heartRateManager.setHeartRate("telemetry", telemetryHeartRate)
        onContactHeartRateChanged: heartRateManager.setHeartRate("contact", contactHeartRate)
    }

    RecoveryManager{
        id: recoveryManager
        onCompleted: {
            errorManager.preventUse = false
            errorManager.degradedUse = false
            popupManager.reset()
        }
    }

    Connections{
        target: errorManager
        onErrorEvent:{
            recoveryManager.errorTriggeredSlot(errorcode)
        }

        onConsoleDetectedFaultRecovered: {
            errorManager.preventUse = false
            errorManager.degradedUse = false
            popupManager.reset()
        }
    }

    function personPresentSupported()
    {
        return ( machineinfo.modelString === "TI1000" ||
                machineinfo.modelString === "TC900" ||
                machineinfo.modelString === "TC650" ||
                machineinfo.modelString === "TC400")
    }

    PersonPresentDetectionManager{
        id:personPresentDetectionManager
        isEnabled: (options.ppdEnabled) && personPresentSupported()
    }

    KeyMapper{
        id: keymapper
        function handleStart() {
            if (isTreadmill && machinecontrol.treadmillSpeedActual !== 0) {
                return
            }

            if(isTreadmill && canStartWorkout == false) {
                return
            }

            if (blockUseMessage.visible || errorManager.fatalError || errorManager.preventUse || main.state === "SleepModeState")
                return
            if (accessDialog.visible)
                return
            if (softwareUpdatePrompt.active)
                return
            if (logUploadPrompt.visible)
                return
            if (main.isTouch) {
                quickStart()
                main.channelSelected = ""
            }
        }
        onStartQuickStartPressed: {
            keymapper.handleStart()
        }
        onStartPressed: {
            keymapper.handleStart()
        }
    }

    Connections {
        id: tftQuickStartHandler
        target: main.isTft && !main.blockKeys && !blockUseMessage.visible && (main.state === "HomeState" || main.state === "MenuState" || main.state === "AllWorkoutsState") ? keymapper : null
        onStartPressed: {
            main.debugKeys("startPressed", "main")
            if(machinecontrol.treadmillSpeedActual == 0 || machineinfo.productTypeString != "Treadmill") {
                quickStart()
            }
        }
        onStartQuickStartPressed: { // thumb quickstart button
            main.debugKeys("startQuickStartPressed", "main")
            if(machinecontrol.treadmillSpeedActual == 0 || machineinfo.productTypeString != "Treadmill") {
                quickStart()
            }
        }
    }


    WorkoutRunData {
       id: aWorkoutRunData

        workoutProgram: workoutProgram.programIndex
        productTypeString: machineinfo.productTypeString
        heartRate: heartRateManager.heartRate
        workloadLevel: workoutManager.workloadLevel
        paceLevel: workoutManager.paceLevel
        wattsAt60: workoutManager.wattsAt60;
        timeFactor: main.timeFactor
        strideLength: workoutManager.strideLevel
    }

    WorkoutManager {
        id: workoutManager

        property bool withLimit: true

        productType: machineinfo.productType
        workoutData: aWorkoutRunData
        userData: userData
        heartRate: heartRateManager.heartRate
        treadSpeedGUI: machinecontrol.treadmillSpeedTarget
        treadInclineGUI: machinecontrol.treadmillInclineTarget

        onWattsAt60Changed: {machinecontrol.workload = wattsAt60; aWorkoutRunData.setWatts(wattsAt60)}
        onSpeedChanged: {
            machinecontrol.treadmillSpeedTarget = (main.state === "RunState") ? treadSpeed : 0 // allow non-zero speed only in RunState
        }
        onInclineChanged: {
            machinecontrol.treadmillInclineTarget = treadIncline
            if(main.lowerSystem === "NUCLEUS") {
                workoutData.incline = treadIncline / 10.0
            }
        }

        onStrideLevelChanged: machinecontrol.strideLengthTarget = strideLevel

        workoutProgram: workoutProgram

        onWorkoutDataChanged: {
            // TODO Send workout data from console to Nucleus for CSAFE purposes
        }

        timeFactor: main.timeFactor
        cooldownDuration: options.cooldownTime
        //treadInclineMax: configurationData.maxInclinePercentGrade
        //treadInclineMin: configurationData.minInclinePercentGrade

        onWorkoutModeChanged:
        {
            var modeStr = ""
            switch(mode) {
            case WorkoutRunData.WARM_UP_MODE: modeStr = "Warm Up"; break;
            case WorkoutRunData.COOL_DOWN_MODE: modeStr = "Cool Down"; break;
            case WorkoutRunData.WORKOUT_MODE: modeStr = "Workout"; break;
            case WorkoutRunData.SUMMARY_MODE: modeStr = "Summary"; break;
            }

            print("WorkoutManager: Workout Mode is " + modeStr)
            aWorkoutRunData.setWorkoutMode(mode);
            if (mode == WorkoutRunData.SUMMARY_MODE) {
                workoutScreen.stopWorkout()
                treadSpeed = 0
                machinecontrol.treadmillSpeedTarget = treadSpeed
            }
            if (mode == WorkoutRunData.COOL_DOWN_MODE) {
                workoutScreen.showMessage(qsTr("COOL DOWN") + application.emptyString)
            }
        }
        onDisplayHRPrompt: workoutScreen.displayHRPrompt(index);
    } // end WorkoutManager

    MessagePopup {
        id: speedInclineLimitedPopup
        blockKeys: false
        isTransparent: true
        Component.onCompleted: hide()
    }

    Connections {
        target: machinecontrol
        onSpeedLimited: {
            workoutManager.treadSpeed = machinecontrol.treadmillSpeedTarget
            var msg = qsTr("Speed limited to ")

            var speed = workoutManager.treadSpeed / 10.0
            if (workoutScreen.unitState === "ENGLISH") {
                msg += speed + " " + qsTr("MPH")
            } else {
                msg += Globals.mileToKm(speed).toFixed(1) + " " + qsTr("KPH")
            }

            speedInclineLimitedPopup.showMessage(msg, 3)
        }
        onInclineLimited: {
            workoutManager.treadIncline = machinecontrol.treadmillInclineTarget
            var msg = qsTr("Incline limited to ") + workoutManager.treadIncline / 10 + " %";
            speedInclineLimitedPopup.showMessage(msg, 3)
        }
        onInclineInhibited: {
            workoutManager.treadIncline = machinecontrol.treadmillInclineTarget
            infoBanner.showMessage("Incline disabled", 5)
        }
    }

    ProgSetup {
        id: setupScreen
        enabled: (visible && opacity > 0.0)

        focus: main.state === "SetupState"

        onStartProgram:{
            workoutProgram.setupComplete()
            workoutStartManager.conditionalStart()
//            workoutManager.start();
//            main.state = "RunState";
//            workoutScreen.start();
        }

        onSaveCustomProgram: workoutProgram.saveProgram(programName,saveLocation);

        anchors.fill: parent
        opacity: 0
        z: 8
    }

    WorkoutProgram {
        id: workoutProgram

        userData: userData
        userWeightMax: {
            if(main.lowerSystem === "NUCLEUS") {
                return nucleus.maxUserWeightLbs
            } else {
                return 400
            }
        }
        metricUnits: options.metricUnits
        saveWorkoutEnabled: options.saveWorkoutEnabled
        property int maxSegments: Math.min( profile1.count, profile2.count)
        inclineTable: machineinfo.inclineTable
        speedTable: machineinfo.speedTable
        speedTableMetric: machineinfo.speedTableMetric
        brakeWattsTable: machineinfo.brakeWattsTable
        paceTable: machineinfo.paceTable
        brainboardProductTypeString: machineinfo.productTypeString
    }

    BlueToothLowEnergy {
        id: ble
    }

    WorkoutScreen {
        id: workoutScreen
        enabled: (visible && opacity > 0.0)

        visible: main.state === "RunState" && !fullscreenVideoFrame.visible

        onWorkoutStopped: {
            audioManager.reset()
            browserAudioWatcher.kill()
            tv.currentChannel = -1
            main.state = "WorkoutSummaryState";
            workoutManager.stop();
            stopPrompt.hide()
            viewContainerBanner.hide()
            if (bluetoothAdapter.discovering) {
                bluetoothAdapter.stopDiscovery()
            }

            workout_summary.setData( aWorkoutRunData )
            //prepare data for blutooth advertisment
            ble.exportWorkoutDataForBleno(aWorkoutRunData,application)

            // add to statistics
            statisticManager.mileage += aWorkoutRunData.totalDistance * 100
            statisticManager.runHours += aWorkoutRunData.totalTime
            statisticManager.totalDistance += aWorkoutRunData.totalDistance * 100
            statisticManager.writeFile()

            // end of workout
            if (netpulseAvailable)
            {
                dataExport.writeNetpulseData("/netpulse/upload")
                netpulse.endWorkout()
                netpulse.logOut()
            }

            workoutManager.reset();
            workoutManager.resetLevels();
            aWorkoutRunData.mode = WorkoutRunData.SUMMARY_MODE
            setupScreen.clearListView();

            workoutScreen.container.destroyViews()

            // if bluetooth is connected, disconnect the bluetooth device
            if(bluetoothAdapter.connected){
                bluetoothAdapter.disconnect()
            } 		
            if (bluetoothAdapter.connectedHeadset)
                bluetoothAdapter.disconnectHeadset()

            machinecontrol.autoStrideEnabled = false

            if(options.triggerWorkoutAutoRestart) {
                console.log("Restarting autostart timer................................................................................. autostart in " + autostartTimer.interval + "ms")
                autostartTimer.restart()
            }
        }//

        onWorkoutPaused: {
            workoutManager.pause()
        }
        onWorkoutStarted: {
            quickStartLoading = false
            if (options.cabEnabled) {
                var firstChannel = tv.channelList.getChannelItem(0)
                if (firstChannel) {
                    console.log("CAB: Autotuning to", firstChannel.id)
                    tv.tuneChannelID(firstChannel.id)
                }
            }

            if(options.triggerWorkoutAutoRestart) {
                console.log("Restarting autostop timer......................................................................................autostop in "  + autostopTimer.interval +"ms.")
                autostopTimer.restart()
                // Set incline to max
                console.log("Automatically Setting incline to " + workoutProgram.maxIncline)
                workoutManager.treadIncline = workoutProgram.maxIncline * 10
            }
        }


        anchors.fill: parent
        opacity: 0


    }

    WorkoutSummary {
        id: workout_summary
        timeLeft: idle_timer.secondsLeft()
        opacity: 0
        enabled: (visible && opacity > 0.0)
        focus: main.state === "WorkoutSummaryState"
        onClose: {
            saveHasHappened = false
            main.state = "HomeState"
        }
        onSave: {
            var success = aWorkoutRunData.saveResults()
            if(success) {
                saveHasHappened = true
                infoPopup.showMessage(qsTr("Summary has been saved.") + application.emptyString, 5);
            }
            else
                infoPopup.showMessage(qsTr("Error. Check your memory device.") + application.emptyString, 5);
        }
    }

    SetupWizard{
        id: setupWizard

        focus: main.state === "SetupWizardState"
        isNucleus: machineinfo.systemConfig === "NUCLEUS" || machineinfo.systemConfig === "NUCBOOT"

        opacity: 0
        enabled: (visible && opacity > 0.0)
        onSetupWizardComplete: {
            main.restartRequired = true
            main.state = "HomeState"
        }
        onLanguageSelected:{
            if(lang){
                application.selectLanguage(lang)
                options.defaultLanguage = lang
                if(netpulseAvailable)
                    netpulse.selectLanguage(lang)

            }
        }

    }

    BlueToothHeartRate {
        id: bluetoothLEAdapter
        onHeartrateChanged: {
            bluetoothLEAdapter.connecting=false
            heartRateManager.setHeartRate("bluetooth", bluetoothLEAdapter.heartrate)
        }
        //onScanningChanged: {
            //if(bluetoothLEAdapter.scanning == true)

        //}
        onHrTimeout: {
            console.log("Heartrate timed out...now disconnecting")
            bluetoothLEAdapter.disconnect(bluetoothLEAdapter.deviceMAC,bluetoothLEAdapter.hrHandle)
        }
        onConnected: {
            heartRateManager.currentBtAddress = bluetoothLEAdapter.deviceMAC
        }
        onHrDisconnectingComplete: {
            heartRateManager.currentBtAddress = ""
            heartRateManager.clearBluetoothHeartRateDeviceList()
            if (bluetoothLEAdapter.nextHrAddress !== "") {
                bluetoothLEAdapter.getHrServiceHandle(bluetoothLEAdapter.nextHrAddress, bluetoothLEAdapter.nextHrAddressType)
                bluetoothLEAdapter.nextHrAddress = ""
                bluetoothLEAdapter.nextHrAddressType = ""
            }
        }
    }


    HeartRateManager{
        id: heartRateManager
        onBluetoothHRConnect: {
            if(heartRateManager.currentBtAddress !== address) {
                if (bluetoothLEAdapter.hrConnected) {
                    // Disconnect them and store address for later reconnect
                    bluetoothLEAdapter.nextHrAddress = address
                    bluetoothLEAdapter.nextHrAddressType = addressType
                    bluetoothLEAdapter.disconnect(bluetoothLEAdapter.deviceMAC, bluetoothLEAdapter.hrHandle)
                    return
                }

                bluetoothLEAdapter.getHrServiceHandle(address, addressType)
            }
        }
    }

    Connections{
        target: workoutScreen
        onWorkoutStopped: heartRateManager.reset()

    }

    // Both SetupWizard and DiagnosticsMain use this, so put it here
    ModalDialog {
        id:productSelectedConfirmation
        property alias text: dialogText.text
        BorderImage {   // pop-up dialog background
            id: border

            x: 332 * main.screenScaleX
            y: 161 * main.screenScaleY
            width: 800 * main.screenScaleX
            height: 350 * main.screenScaleY
            source: main.graphicsPath + "Pop_Up_Lrg_Grey.png"
            border { top: 152 * main.screenScaleY; bottom: 165 * main.screenScaleY; left: 145 * main.screenScaleX; right: 144 * main.screenScaleX }

            Text {
                id: dialogText
                anchors.fill: parent
                anchors.verticalCenter: parent.verticalCenter
                color: "#ffffff"
                anchors.horizontalCenter: parent.horizontalCenter
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                wrapMode: Text.WordWrap
                font.bold: true
                font.pixelSize: 50 * main.screenScaleY

            }
        }
        Component.onCompleted: hide()
        onTimedOut: hide()
    }

    MediaBrowser {
        id: ipod_window

        keysAccepted: opacity > 0
        allowFullscreen: false
        z:4
        opacity: ipod_button.isDown || (!isTouch && home_screen.ipodSelected) ? 1.0 : 0.0
        enabled: (visible && opacity > 0.0)
        visible: opacity > 0

        isDialogVisible: visible

        Connections {
            target: main.isTft && !main.blockKeys && ipod_window.opacity > 0 ? keymapper : null
            onBackPressed: {
                main.debugKeys("backPressed", "main", "ipod_window")
                ipod_window.close()
            }
        }

        onClose: {
            if (main.isTouch)
                ipod_button.setDown(false)
            else if (main.isTft)
                home_screen.setIPodButton(false)
        }
        onVisibleChanged: {
            if(visible)
                shown()
            else
                hidden()
        }
        anchors {   horizontalCenter: parent.horizontalCenter
                    top: parent.top
                    topMargin: 140 * main.screenScaleY
        }

        menu_bottom_margin: ( isTouch ? 60 : 160 ) * main.screenScaleY
        Behavior on opacity { NumberAnimation { duration: 200; easing.type: Easing.InOutQuad } }
    }

    Connections {
        // Workout Summary has a button for resetting the timer, so ignore that state
        target: main.isTft && main.state !== "WorkoutSummaryState" ? keymapper : null
        onDownButtonPressed: { idle_timer.restart(); if (screensaver.visible) screensaver.stop(); }
        onUpButtonPressed: { idle_timer.restart(); if (screensaver.visible) screensaver.stop(); }
        onLeftButtonPressed: { idle_timer.restart(); if (screensaver.visible) screensaver.stop(); }
        onRightButtonPressed: { idle_timer.restart(); if (screensaver.visible) screensaver.stop(); }
        onEnter2Pressed: { idle_timer.restart(); if (screensaver.visible) screensaver.stop(); }
    }

    Loader {
        id: diagnostics_screen

        focus: main.state === "DiagnosticState"
        source: (main.state === "DiagnosticState" ? "Diagnostics/DiagnosticsMain.qml" : "")
        opacity: 0
        enabled: (visible && opacity > 0.0)
        anchors.fill: parent

        Connections {
            target: diagnostics_screen.item
            ignoreUnknownSignals: true

            onClose: {
                main.state = "HomeState"
                if(machinecontrol.isCalibrated){
                    machinecontrol.treadmillInclineTarget = 0
                }
            }
        }
    }

    LogUploader {
        id: logUploader
    }

    LogUploadPrompt {
        id: logUploadPrompt
    }


    Options {
        id: options
        objectName: "options"
        onContactHRPriorityChanged: {
            heartRateManager.setContactPriority(contactHRPriority)
        }
        onLcdBrightnessChanged: main.lcdBrightness = lcdBrightness
        onCabEnabledChanged: {
            nucleus.cabSetEnable(cabEnabled)
        }
        onIgnoreFaultsChanged: {
            if (options.ignoreFaults) {
                popupManager.reset()
            }
        }
        onLoggingEnabledChanged: {
            logger.enabled = loggingEnabled
        }
        onDisableBuzzerChanged: {
            buzzer.setBuzzerEnabled(!options.disableBuzzer)
        }
    }

    property int lcdBrightness: options.lcdBrightness
    onLcdBrightnessChanged: {
        main.setLcdBrightness(lcdBrightness)
    }

    function setLcdBrightness(percent) {
         lcdBrightnessProcess.setBrightness(percent)
    }

    Process {
        id: lcdBrightnessProcess
        program: "scripts/set_brightness.sh"

        function setBrightness(percent) {
            lcdBrightnessProcess.arguments = [ (percent*6000/100).toFixed(0) ]
            lcdBrightnessProcess.start()
        }
    }

    Binding {
        target: application
        property: "showCursor" ; value: options.showCursor
    }

    // Idle timer
    // Time out interval is set by state transition table below.
    // The timer is reset by a screen touch (event sent by application object.)
    // Timeout action is defined in onTriggered event.  Many screens return to HomeScreen state.
    Timer {
        id: idle_timer

        repeat: true
        running: true
        interval: 10000
        onRunningChanged: idleSecondsTimer.reset()
        onIntervalChanged: idleSecondsTimer.reset()

        function secondsLeft() {
            var sec = (interval / 1000) - idleSecondsTimer.seconds
            if(sec < 0 )
                return 0
            else
                return sec
        }

        onTriggered: {
            switch(main.state) {
                case "HomeState":{  // idle timeout on home-screen goes to screen-saver or sleep
                    // close warning screen
                    if (main.isTouch) {
                        home_screen.showingWarningGuide = false
                    }
                    // close TV and iPod windows
                    tv_button.setDown(false)
                    //ipod_button.setDown(false)
                    netpulse.logOut()

                    if(options.screenSaveTimeout) {  // screensaver enabled
                        main.state = "ScreenSaverState"
                        screensaver.start()
                    } // otherwise, go to sleep mode if enabled
                    else if(options.sleepModeTimeout) {
                        main.state = "SleepModeState"
                    }

                    if (remoteAccess.isEnabled) {
                        // Exit screensaver immediately
                        screensaver.stop()
                    }

                    // disconnect bluetooth device if connected
                    if (bluetoothAdapter.connected) {
                        bluetoothAdapter.disconnect()
                    }
                    if (bluetoothAdapter.connectedHeadset)
                        bluetoothAdapter.disconnectHeadset()

                    if(!options.screenSaveTimeout && !options.sleepModeTimeout && options.erpEnabled){
                        erpSleep()
                    }
                    break;
                }
                case "LoginState":
                case "RegisterState":
                {
                    // the login window has its own timeout timer
                    // TODO: should there be a watchdog here in case of communication loss?
                   break;
                }
                case "WorkoutSummaryState": {
                    workout_summary.close() // This will make sure the bt advertise dialog closes on timeou
                    break
                }
                case "MenuState":
                case "AllWorkoutsState":
                case "SetupState":
                case "LanguageMenuState": {
                    // disconnect bluetooth device if connected
                    if (bluetoothAdapter.connected) {
                        bluetoothAdapter.disconnect()
                    }
                    if (bluetoothAdapter.connectedHeadset)
                        bluetoothAdapter.disconnectHeadset()
                    main.state = "HomeState";
                    break;
                }
                case "ScreenSaverState": {
                    if (options.sleepModeTimeout) {
                        main.state = "SleepModeState"
                    } else if(options.erpEnabled){
                        erpSleep()
                    }
                    break;
                }
                case "DiagnosticState":
                case "SetupWizardState":
                {
                    break;
                }
                case "SleepModeState":
                {
                    if(options.erpEnabled){
                        erpSleep()
                    }

                    break;
                }
            } // end switch
        } // end onTriggered
    }

    Timer {
        id: dim_timer
        interval: options.dimDisplayTimeout * 1000
        running: false
        repeat: false
        onTriggered: {
            if (main.state != "HomeState" && main.state != "ScreenSaverState")
                return

            // Set to 40% of brightness setting
            var brightnessLevel = parseInt(0.40 * options.lcdBrightness)
            main.lcdBrightness = brightnessLevel
            console.log("Dim timer fired. Set brightness to", brightnessLevel)
        }
    }

    Timer {
        id:idleSecondsTimer

        property int seconds: 0

        function reset() {
            idleSecondsTimer.seconds = 0
            idleSecondsTimer.restart()
        }

        interval: 1000
        running: true
        onRunningChanged: seconds = 0
        repeat: true
        onTriggered: {
            if(machinecontrol.treadmillSpeedActual > 0 && main.state !== "RunState" && main.state !== "DiagnosticState"){
                machinecontrol.treadmillSpeedTarget = 0
            }
            seconds++
        }
    }

    Timer {
        id: playBluetooth
        interval: 1000; repeat: false; running: false

        onTriggered: {
            if (bluetoothAdapter.currentConnectedDevice) {
                if (bluetoothAdapter.currentConnectedDevice.player) {
                    bluetoothAdapter.currentConnectedDevice.player.play()
                }
            }
        }
    }

    TrueBlueAdapter{
        id: bluetoothAdapter

        onConnectedChanged: {
            if (connected) {
                btMute.setMute(0)
                audioManager.setVolume(audioManager.volumeDefault)
                playBluetooth.start()
            }
        }

        onConnectedHeadsetChanged: {
            if (tv.isIpodInput) {
                // restart tv if iPod is currently playing, so that it will start streaming the ipod
                // input to pulseaudio, which is required for the headset to function
                tv.mrl = ""
                tv.tuneChannelID("IPODINPUT")
            }
        }
    }
    // Expire bluetooth connection after 5 minutes without starting a workout
    Timer {
        id: bluetoothSessionTimer
        running: (main.state !== "RunState") && bluetoothAdapter.connected
        interval: 2 * 60 * 1000
        onTriggered: {
            ipod_window.close()
            if (bluetoothAdapter.connected)
                bluetoothAdapter.disconnect()
            if (bluetoothAdapter.connectedHeadset)
                bluetoothAdapter.disconnectHeadset()
        }
    }

    // Expire bluetooth Heartrate connection after 5 minutes without starting a workout
    Timer {
        id: bluetoothLESessionTimer
        running: (main.state !== "RunState") && bluetoothLEAdapter.hrConnected
        interval: 2 * 60 * 1000
        onTriggered: {
            if (bluetoothLEAdapter.hrConnected)
                bluetoothLEAdapter.disconnect()
        }
    }

    KeypadProcess {
        id: urlKeypad
        name: "urlKeypad"
    }

    KeypadProcess {
        id: inputKeypad
        name: "inputKeypad"
    }

    Rectangle {
        id: touchSplat

        property int size: 10

        opacity: 0.0 // Hide until first touch
        width: size
        height: width
        radius: width
        z: 100

        transform: Translate { x:-touchSplat.size/2 ; y: -touchSplat.size/2 }

        color: "#0099FF"

        ParallelAnimation {
            id:splat
            NumberAnimation { target: touchSplat; properties: "size" ; from: 15 ; to: 0; duration: 500; easing.type: Easing.InOutQuad }
            NumberAnimation { target: touchSplat; properties: "opacity" ; from: 0.3 ; to: 0; duration: 500; easing.type: Easing.InOutQuad }
        }
    }

    Connections {
        // reset the idle timer when there's a screen touch
        target: application
        onScreenTouched: {
            splat.stop() // Stop in-progress splat
            idle_timer.restart()
            lang_timer.restart()
            touchSplat.x = x
            touchSplat.y = y
            touchSplat.opacity = 0.3
            touchSplat.size = 15
            if(main.state == "SleepModeState")
                main.state = "HomeState"
            else if (main.state === "HomeState") {
                main.lcdBrightness = options.lcdBrightness
                if (!options.screenSaveTimeout && options.dimDisplayTimeout)
                    dim_timer.restart()
            }
        }
        onMouseReleased: {
            touchSplat.x = x
            touchSplat.y = y
            splat.restart()
        }
    }

    Connections {
        target: machineinfo
        onDetectionComplete: {
            main.lowerSystem = lowerSystem
            if(lowerSystem === "NO_LOWERSYSTEM" && !options.ignoreFaults){
                blockUseMessage.hide()
                errorManager.errorReceived(ErrorManager.ERRORNO_CONSOLE_USBCONNECTIONTERMINATED,statisticManager.totalDistance)
            }

            if (main.lowerSystem === "NUCLEUS" || main.lowerSystem === "NUCBOOT") {
                if (!options.setupWizardCompleted) {
                    main.state = "SetupWizardState"
                }
            }

        }
    }

    onStateChanged: {
        print("Main: UI state = " + main.state)

        if(state === "HomeState"){
            if(machineinfo.productTypeString === "Elliptical") {
                machinecontrol.workload = 0
            }
        }
    }

    Process {
        id: paVolControl
        program: "scripts/true-pa-volume-control.sh"
        property bool isMute
        property int _currentVolSetting: 0

        function setVolume(vol_setting){

            /* Only run the script if the vol setting is different */
            if(_currentVolSetting === vol_setting) {
                return
            }

            _currentVolSetting = vol_setting

            var vol = (6554 * vol_setting) + 6550
            var volStr = vol.toString();
            paVolControl.arguments = ["vol", volStr]
            paVolControl.start()
        }

        function mute() {
            paVolControl.arguments = ["mute"]
            paVolControl.start()
        }

        function info() {
            paVolControl.arguments = ["info"]
            paVolControl.start()
        }

        function parseOutput(str){
            var strList = str.split(" ")
            if(strList[0] === "info"){
                var volumeInt = parseInt(strList[2])
                var isMuteStr = strList[3]

                if(isMuteStr === "yes") {
                    isMute = true
                } else if(isMuteStr === "no") {
                    isMute = false
                }

                console.debug(volumeInt + " " + isMuteStr)
            }
        }

        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim()
            console.log("output from paVolControl:",data)
            parseOutput(data)
        }
    }

    Timer {
        id: check_nucleus_faults_timer
        interval:24*3600*1000
        running: true
        repeat: true
        onTriggered: {
            // Retrieve Nucleus error logs
            console.log("Retrieving Nucleus fault log")
            errorManager.retrieveFaultLogs()
         }
    }

    states: [
        State {
            name: "HomeState"
            PropertyChanges {target: home_screen_loader ; opacity: 1 }
            PropertyChanges {target: screen_lock_button ; opacity: isTouch ? 1 : 0 }
            PropertyChanges {target: navbar; opacity: 1 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ;  opacity: 1 }
            PropertyChanges {target: language_button ; opacity: 1 }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 1   }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: warningText ; opacity: 1 }
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            // set timeout for screen-saver or dim mode or sleep mode
            PropertyChanges {target: idle_timer ;
                interval: {
                    if(options.screenSaveTimeout)
                        return options.screenSaveTimeout*1000
                    else if(options.sleepModeTimeout)
                        return options.sleepModeTimeout*1000
                    else if(options.erpEnabled && options.erpTimeout)
                        return options.erpTimeout*1000
                    else
                        return 0
                }
            }
            PropertyChanges { target: dim_timer; running: options.dimDisplayTimeout > 0 && !options.screenSaveTimeout }
            PropertyChanges {target: workout_summary ; dialogVisible: false }
        },
        State {
            name: "LoginState"
            PropertyChanges {target: home_screen_loader ; opacity: 1 }
            // netpulse
            StateChangeScript { script: netpulse.showPage("login") }
            PropertyChanges {target: screen_lock_button ; opacity: isTouch ? 1 : 0 }
            PropertyChanges {target: navbar; opacity: 1 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ;  opacity: 1 }
            PropertyChanges {target: language_button ; opacity: 1 }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 1   }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: warningText ; opacity: 0 }
            // currently the idle timer does nothing in this state.
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: 30000}
            PropertyChanges {target: dim_timer; running: false }
        },
        State {
            name: "RegisterState"
            PropertyChanges {target: home_screen_loader ; opacity: 1 }
            // netpulse
            StateChangeScript { script: netpulse.showPage("registration") }
            PropertyChanges {target: screen_lock_button ; opacity: isTouch ? 1 : 0 }
            PropertyChanges {target: navbar; opacity: 1 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ;  opacity: 1 }
            PropertyChanges {target: language_button ; opacity: 1 }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 1   }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: warningText ; opacity: 0 }
            // currently the idle timer does nothing in this state.
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: 30000}
            PropertyChanges {target: dim_timer; running: false }
        },
        State {
            name: "LanguageMenuState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: screen_lock_button ; opacity: 0 }
            PropertyChanges {target: navbar; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 1 }
            PropertyChanges {target: workout_menu_loader; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ;  opacity: 0 }
            PropertyChanges {target: language_button ; opacity: 0 }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 0   }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: warningText ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: options.workoutMenuTimeout * 1000}
            PropertyChanges {target: dim_timer; running: false }
        },

        State {
            name: "MenuState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: screen_lock_button ; opacity: 0 }
            PropertyChanges {target: navbar; opacity: 1 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 1 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ; opacity: 1 }
            PropertyChanges {target: language_button ; opacity: 1 }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 1 }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: warningText ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: options.workoutMenuTimeout * 1000}
            PropertyChanges {target: dim_timer; running: false }
        },
        State {
            name: "AllWorkoutsState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: screen_lock_button ; opacity: 0 }
            PropertyChanges {target: navbar; opacity: 1 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 1 }
            PropertyChanges {target: side_panel ; opacity: 0 }
            PropertyChanges {target: language_button ; opacity: 1 }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 1 }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: warningText ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: options.workoutMenuTimeout * 1000}
            PropertyChanges {target: dim_timer; running: false }
        },
        State {
            name: "LoadState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: screen_lock_button ; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ; opacity: 0 }
            PropertyChanges {target: language_button ; opacity: 0 }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 0 }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: loadingProgramNotice ; opacity: 1 }
            PropertyChanges {target: warningText ; opacity: 0 }
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
        },
        State {
            name: "SetupState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: screen_lock_button ; opacity: 0 }
            PropertyChanges {target: navbar; opacity: 1 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ; opacity: 0 }
            PropertyChanges {target: language_button ; opacity: 0 }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 0 }
            PropertyChanges {target: setupScreen ; opacity: 1 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: warningText ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: options.workoutSetupTimeout * 1000}
            PropertyChanges {target: dim_timer; running: false }
        },
        State {
            name: "RunState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: screen_lock_button ; opacity: 0 }
            PropertyChanges {target: navbar; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ; opacity: 0 }
            PropertyChanges {target: language_button ; opacity: 0 }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 0 }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 1 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: warningText ; opacity: 0 }
            // turn off idle timer - run screen will have its own
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: 0}
            PropertyChanges {target: dim_timer; running: false }
        },
        State {
            name: "WorkoutSummaryState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: screen_lock_button ; opacity: 0 }
            PropertyChanges {target: navbar; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ;  opacity: 0 }
            PropertyChanges {target: language_button ; opacity: 0 }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 0   }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 1 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: warningText ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: options.workoutSummaryTimeout * 1000}
            PropertyChanges {target: dim_timer; running: false }

        },
        State {
            name: "DiagnosticState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: screen_lock_button ; opacity: 0 }
            PropertyChanges {target: navbar; opacity: 1 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ; opacity: 0 }
            PropertyChanges {target: language_button ; opacity: 0 }
            PropertyChanges {target: diagnostics_screen ; opacity: 1 ; z: 50}
            PropertyChanges {target: logoImage ; opacity: 0 }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0  }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: warningText ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: 300000}
            PropertyChanges {target: dim_timer; running: false }

        },
        State {
            name: "SetupWizardState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: screen_lock_button ; opacity: 0 }
            PropertyChanges {target: navbar; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: workouts_all ; opacity: 0 }
            PropertyChanges {target: side_panel ; opacity: 0 }
            PropertyChanges {target: language_button ; opacity: 0 }
            PropertyChanges {target: diagnostics_screen ; opacity: 0 ; z: 0}
            PropertyChanges {target: logoImage ; opacity: 0 }
            PropertyChanges {target: setupScreen ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0  }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 1 }
            PropertyChanges {target: warningText ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: 0}
            PropertyChanges {target: dim_timer; running: false }

        },
        State {
            name: "ScreenSaverState"
             // set timeout until moving from screensaver to dim-mode, sleep-mode, or power-down
            PropertyChanges {target: idle_timer ; interval: options.sleepModeTimeout ? options.sleepModeTimeout * 1000 : options.erpTimeout * 1000 }
            PropertyChanges { target: dim_timer; running: options.dimDisplayTimeout > 0 }
        },
        State {
            name: "SleepModeState"
             // set timeout until moving from screensaver to power-down/sleep mode
            PropertyChanges {target: main; lcdBrightness: 0; restoreEntryValues: true ; explicit: true }
            PropertyChanges {target: idle_timer ; interval: options.erpTimeout * 1000 }
            PropertyChanges {target: dim_timer; running: false }
        }

    ] // end states
    transitions: [
        Transition {
                NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
        },

        Transition {
            from: "HomeState"
            to: "MenuState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    navbar.push(main.graphicsPath + "navbar/Navigation_Home", "HomeState", undefined )
                    workout_menu.changeMenu(main.topmenu) //Menus.menu_elliptical_top)
                }
            }
        },
        Transition {
            from: "HomeState"
            to: "AllWorkoutsState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    navbar.push(main.graphicsPath + "navbar/Navigation_Home", "HomeState", undefined )
                    // refresh menu
                    selectWorkoutMenu()
                    workout_menu.changeMenu(main.allWorkoutsMenu)
                }
            }
        },
        Transition {
            from: "MenuState"
            to: "HomeState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    application.selectLanguage(options.defaultLanguage) // reset language after workout
                    if(netpulseAvailable)
                        netpulse.setLanguage(options.defaultLanguage)
                    navbar.clear()
                    netpulse.hideAll();
                    clearQLists()
                    main.quickStartLoading = false
                }
            }
        },
        Transition {
            from: "AllWorkoutsState"
            to: "HomeState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    application.selectLanguage(options.defaultLanguage) // reset language after workout
                    if(netpulseAvailable)
                        netpulse.setLanguage(options.defaultLanguage)
                    navbar.clear()
                    netpulse.hideAll();
                    clearQLists()
                    main.quickStartLoading = false
                }
            }
        },
        Transition {
            from: "SetupState"
            to: "HomeState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    application.selectLanguage(options.defaultLanguage) // reset language after workout
                    if(netpulseAvailable)
                        netpulse.setLanguage(options.defaultLanguage)
                    navbar.clear()
                    netpulse.hideAll();
                    clearQLists()
                    main.quickStartLoading = false
                }
            }
        },
        Transition {
            from: "DiagnosticState"
            to: "HomeState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    application.selectLanguage(options.defaultLanguage) // reset language after workout
                    if(netpulseAvailable)
                        netpulse.setLanguage(options.defaultLanguage)
                    navbar.clear()
                    netpulse.hideAll();
                    clearQLists()
                    main.quickStartLoading = false
                    syncDisk.start()
                    if(machinecontrol.treadmillSpeedActual > 0 && main.state != "RunState"){
                        machinecontrol.treadmillSpeedTarget = 0;
                    }

                    if (main.isTft) home_screen.resetCursor()
                }
            }
        },
        Transition {
            from: "HomeState"
            to: "DiagnosticState"
            ScriptAction {
                script: softwareUpdatePrompt.hide()
            }
        },
        Transition {
            from: "MenuState"
            to: "AllWorkoutsState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: { navbar.push(main.graphicsPath + "navbar/Navigation_WF", "MenuState", undefined ) }
            }
        },
        Transition {
            from: "WorkoutSummaryState"
            to: "HomeState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    workout_summary.saveHasHappened = false
                    application.selectLanguage(options.defaultLanguage) // reset language after workout
                    if(netpulseAvailable)
                        netpulse.setLanguage(options.defaultLanguage)
                    navbar.clear()
                    netpulse.hideAll();
                    clearQLists()
                    main.quickStartLoading = false
                    if(machinecontrol.treadmillSpeedActual > 0 ){
                        machinecontrol.treadmillSpeedTarget = 0;
                    }
                    if (main.isTft) home_screen.resetCursor()
                }
            }
        }
    ]
}
